from insights.parsers import SkipException
from insights.parsers import passenger_status
from insights.parsers.passenger_status import PassengerStatus
from insights.tests import context_wrap
import pytest
import doctest


PASS_STATUS = """
Version : 4.0.18
Date    : 2018-10-23 15:42:04 +0800
Instance: 1265
----------- General information -----------
Max pool size : 12
Processes     : 2
Requests in top-level queue : 0

----------- Application groups -----------
/usr/share/foreman#default:
  App root: /usr/share/foreman
  Requests in queue: 192
  * PID: 30131   Sessions: 1       Processed: 991     Uptime: 2h 9m 8s
    CPU: 3%      Memory  : 562M    Last used: 1h 53m 51s
  * PID: 32450   Sessions: 1       Processed: 966     Uptime: 2h 8m 15s
    CPU: 4%      Memory  : 463M    Last used: 1h 48m 17
  * PID: 4693    Sessions: 1       Processed: 939     Uptime: 2h 6m 32s
    CPU: 3%      Memory  : 470M    Last used: 1h 50m 48

/etc/puppet/rack#default:
  App root: /etc/puppet/rack
  Requests in queue: 0
  * PID: 21934   Sessions: 1       Processed: 380     Uptime: 1h 33m 34s
    CPU: 1%      Memory  : 528M    Last used: 1h 29m 4
  * PID: 26194   Sessions: 1       Processed: 544     Uptime: 1h 31m 34s
    CPU: 2%      Memory  : 490M    Last used: 1h 23m 5
  * PID: 32384   Sessions: 1       Processed: 36      Uptime: 1h 0m 29s
    CPU: 0%      Memory  : 561M    Last used: 1h 0m 3s
"""

PASS_STATUS_SP = """
Version : 4.0.18
Date    : 2019-06-10 03:17:38 +0100
Instance: 14745
----------- General information -----------
Max pool size : 60
Processes     : 7
Requests in top-level queue : 0

----------- Application groups -----------
/usr/share/foreman#default:
  App root: /usr/share/foreman
  Requests in queue: 0
  * PID: 39176  Sessions: 0     Processed: 194     Uptime: 24h 9m 0s
    CPU: 0%     Memory  : 488M  Last used: 20m 24s a
  * PID: 39342  Sessions: 0       Processed: 0       Uptime: 24h 8m 58s
    CPU: 0%     Memory  : 178M    Last used: 24h 8m 5
  * PID: 39377  Sessions: 0       Processed: 0       Uptime: 24h 8m 58s
    CPU: 0%     Memory  : 179M    Last used: 24h 8m 5
  * PID: 39478  Sessions: 0       Processed: 0       Uptime: 24h 8m 57s
    CPU: 0%     Memory  : 178M    Last used: 24h 8m 5
  * PID: 39525  Sessions: 0       Processed: 0       Uptime: 24h 8m 57s
    CPU: 0%     Memory  : 173M    Last used: 24h 8m 5
  * PID: 39614  Sessions: 0       Processed: 0       Uptime: 24h 8m 56s
    CPU: 0%     Memory  : 174M    Last used: 24h 8m 5

/etc/puppet/rack#default:
  App root: /etc/puppet/rack
  Requests in queue: 0
  * PID: 39667   Sessions: 0       Processed: 241     Uptime: 24h 8m 56s
    CPU: 0%      Memory  : 45M     Last used: 20m 24s
"""

PASS_STATUS_EXP1 = """
wrong content
"""


def test_passenger_status():
    passenger_status = PassengerStatus(context_wrap(PASS_STATUS))
    assert passenger_status["Version"] == '4.0.18'
    assert len(passenger_status['foreman_default']['p_list']) == 3
    assert 'rack_default' in passenger_status


def test_passenger_status_2():
    passenger_status = PassengerStatus(context_wrap(PASS_STATUS_SP))
    assert passenger_status["Version"] == '4.0.18'
    assert len(passenger_status['foreman_default']['p_list']) == 6
    foreman_default_p_list = passenger_status['foreman_default']['p_list']
    assert foreman_default_p_list[0] == {
            'PID': '39176', 'Sessions': '0', 'Processed': '194', 'Uptime': '24h 9m 0s',
            'CPU': '0%', 'Memory': '488M', 'Last used': '20m 24s a'}
    assert foreman_default_p_list[1] == {
            'PID': '39342', 'Sessions': '0', 'Processed': '0', 'Uptime': '24h 8m 58s',
            'CPU': '0%', 'Memory': '178M', 'Last used': '24h 8m 5'}
    assert foreman_default_p_list[-1] == {
            'PID': '39614', 'Sessions': '0', 'Processed': '0', 'Uptime': '24h 8m 56s',
            'CPU': '0%', 'Memory': '174M', 'Last used': '24h 8m 5'}
    assert 'rack_default' in passenger_status


def test_passenger_status_ex():
    with pytest.raises(SkipException):
        PassengerStatus(context_wrap(PASS_STATUS_EXP1))


def test_passenger_status_doc_examples():
    env = {
        'passenger_status': PassengerStatus(context_wrap(PASS_STATUS)),
    }
    failed, total = doctest.testmod(passenger_status, globs=env)
    assert failed == 0



"""
Software Collections list output - command ``scl --list`` RHEL-6/7
==================================================================

This module provides parser for list output of ``scl``. This spec
is valid for ``RHEL-6/7`` only, ``-l|--list`` is ``deprecated`` in
``RHEL-8``. On ``RHEL-8`` same functionality can be achieved by
``scl list-collections`` spec.

Parser provided by this module is:

SoftwareCollectionsListInstalled - command ``scl --list``
---------------------------------------------------------

"""

from insights import CommandParser, parser
from insights.parsers import get_active_lines
from insights.specs import Specs
from insights.components.rhel_version import IsRhel6, IsRhel7


@parser(Specs.software_collections_list, [IsRhel6, IsRhel7])
class SoftwareCollectionsListInstalled(CommandParser):
    """
    An object for parsing the output of ``scl --list``.

    Sample input file::

        devtoolset-7
        httpd24
        python27
        rh-mysql57
        rh-nodejs8
        rh-php71
        rh-python36
        rh-ruby24

    Examples:
        >>> type(collections)
        <class 'insights.parsers.software_collections_list.SoftwareCollectionsListInstalled'>
        >>> len(collections.records)
        8
        >>> coll1 = collections.records[0]
        >>> coll1
        'devtoolset-7'
        >>> coll2 = collections.records[4]
        >>> coll2
        'rh-nodejs8'
        >>> collections.exists('rh-ruby24')
        True
        >>> collections.exists('rh-missing_colection')
        False

    """
    def parse_content(self, content):
        self.records = []
        for line in get_active_lines(content):
            self.records.append(line)

    def exists(self, collection_name):
        """
        Checks if the collection is installed on the system.

        Args:
           service_name (str): collections name

        Returns:
            bool: True if collections exists, False otherwise.
        """
        return collection_name in self.records



"""
DnsmasqConf - files ``/etc/dnsmasq.conf`` and ``/etc/dnsmasq.d/*.conf``
=======================================================================
"""

import string

from insights.core.plugins import parser
from insights.core import ConfigParser
from insights.specs import Specs

from insights.parsr import (Char, EOF, Forward, Lift, LineEnd, Many, Number,
        OneLineComment, Opt, PosMarker, skip_none, String, WS, WSChar)
from insights.parsr.query import Directive, Entry


@parser(Specs.dnsmasq_config)
class DnsmasqConf(ConfigParser):
    """
    Class to parses the content of dnsmasq configuration files ``/etc/dnsmasq.conf`` and ``/etc/dnsmasq.d/*.conf``

    Sample configuration output::

        # Listen on this specific port instead of the standard DNS port
        # (53). Setting this to zero completely disables DNS function,
        # leaving only DHCP and/or TFTP.
        port=5353

        no-resolv
        domain-needed
        no-negcache
        max-cache-ttl=1
        enable-dbus
        dns-forward-max=5000
        cache-size=5000
        bind-dynamic
        except-interface=lo
        server=/in-addr.arpa/127.0.0.1
        server=/cluster.local/127.0.0.1
        # End of config

    Examples::
        >>> "no-resolv" in conf
        True
        >>> conf["server"]
        server=/in-addr.arpa/127.0.0.1
        server=/cluster.local/127.0.0.1
        >>> conf["dns-forward-max"][-1].value
        5000

    """

    def parse_doc(self, content):
        return parse_doc("\n".join(content), ctx=self)


def parse_doc(f, ctx=None, line_end="\n"):
    def to_entry(name, rest):
        rest = [] if not rest else [rest]
        return Directive(name=name.value, attrs=rest, lineno=name.lineno, src=ctx)

    Sep = Char("=")
    Stmt = Forward()
    Num = Number & (WSChar | LineEnd)
    Comment = (WS >> OneLineComment("#").map(lambda x: None))
    Bare = String(set(string.printable) - (set("#\n\r")))
    Name = WS >> PosMarker(String(string.ascii_letters + "_-" + string.digits)) << WS
    Value = WS >> (Num | Bare) << WS
    Stanza = (Lift(to_entry) * Name * (Opt(Sep >> Value))) | Comment
    Stmt <= WS >> Stanza << WS
    Doc = Many(Stmt).map(skip_none)
    Top = Doc + EOF

    return Entry(children=Top(f)[0], src=ctx)



from insights.parsers import openshift_hosts
from insights.tests import context_wrap


OPENSHIFTHOSTS = """
[OSEv3:children]
nodes
nfs
masters
etcd

[BlankTest]

[OSEv3:vars]
openshift_master_cluster_public_hostname=None
ansible_ssh_user=root
openshift_master_cluster_hostname=None
openshift_hostname_check=false
deployment_type=openshift-enterprise

[nodes]
master.ose35.com  openshift_public_ip=192.66.208.202 openshift_ip=192.66.208.202 openshift_public_hostname=master.ose35.com openshift_hostname=master.ose35.com connect_to=master.ose35.com openshift_schedulable=False ansible_connection=local
node1.ose35.com  openshift_public_ip=192.66.208.169 openshift_ip=192.66.208.169 openshift_public_hostname=node1.ose35.com openshift_hostname=node1.ose35.com connect_to=node1.ose35.com openshift_node_labels="{'region': 'infra','zone': 'default'}" openshift_schedulable=True
node2.ose35.com  openshift_public_ip=192.66.208.170 openshift_ip=192.66.208.170 openshift_public_hostname=node2.ose35.com openshift_hostname=node2.ose35.com connect_to=node2.ose35.com openshift_node_labels="{'region': 'infra','zone': 'default'}" openshift_schedulable=True

[nfs]
master.ose35.com  openshift_public_ip=192.66.208.202 openshift_ip=192.66.208.202 openshift_public_hostname=master.ose35.com openshift_hostname=master.ose35.com connect_to=master.ose35.com ansible_connection=local

[masters]
master.ose35.com  openshift_public_ip=192.66.208.202 openshift_ip=192.66.208.202 openshift_public_hostname=master.ose35.com openshift_hostname=master.ose35.com connect_to=master.ose35.com ansible_connection=local

[etcd]
master.ose35.com  openshift_public_ip=192.66.208.202 openshift_ip=192.66.208.202 openshift_public_hostname=master.ose35.com openshift_hostname=master.ose35.com connect_to=master.ose35.com ansible_connection=local
""".strip()


def test_openshifthosts():
    host_info = openshift_hosts.OpenShiftHosts(context_wrap(OPENSHIFTHOSTS))
    assert host_info["OSEv3:children"] == ["nodes", "nfs", "masters", "etcd"]
    assert host_info["OSEv3:vars"]["ansible_ssh_user"] == "root"
    assert host_info["nodes"]["master.ose35.com"]["openshift_public_ip"] == "192.66.208.202"
    assert host_info["nodes"]["node1.ose35.com"]["openshift_node_labels"] == {'region': 'infra', 'zone': 'default'}
    assert host_info.has_node("node1.ose35.com")
    assert not host_info.has_var("openshift_use_crio")
    assert host_info.has_node_type("etcd")
    assert host_info["BlankTest"] == {}



"""
dmsetup commands - Command ``dmsetup``
======================================

Parsers for parsing and extracting data from output of commands related to
``dmsetup``.
Parsers contained in this module are:

DmsetupInfo - command ``dmsetup info -C``
-----------------------------------------

"""
from collections import namedtuple
from insights import parser, CommandParser
from insights.parsers import parse_delimited_table
from insights.specs import Specs


SetupInfo = namedtuple('SetupInfo', [
    'name', 'major', 'minor', 'open', 'segments', 'events',
    'live_table', 'inactive_table', 'suspended', 'readonly', 'uuid']
)
""" Data structure to represent dmsetup information """


@parser(Specs.dmsetup_info)
class DmsetupInfo(CommandParser):
    """
    ``dmsetup info -C`` command output

    Example input::

        Name               Maj Min Stat Open Targ Event  UUID
        VG00-tmp           253   8 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4
        VG00-home          253   3 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxCqXOnbGe2zjhX923dFiIdl1oi7mO9tXp
        VG00-var           253   6 L--w    1    2      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxicvyvt67113nTb8vMlGfgdEjDx0LKT2O
        VG00-swap          253   1 L--w    2    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax3Ll2XhOYZkylx1CjOQi7G4yHgrIOsyqG
        VG00-root          253   0 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxKpnAKYhrYMYMNMwjegkW965bUgtJFTRY
        VG00-var_log_audit 253   5 L--w    1    1      0 LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTaxwQ8R0XWJRm86QX3befq1cHRy47Von6ZW

    Example data structure produced::

        data = [
          {
            'Stat': 'L--w',
            'Name': 'VG00-tmp',
            'Min': '8',
            'Targ': '1',
            'Maj': '253',
            'Open': '1',
            'Event': '0',
            'UUID': 'LVM-gy9uAwD7LuTIApplr2sogbOx5iS0FTax6lLmBji2ueSbX49gxcV76M29cmukQiw4'
          },...
        ]

    Attributes:
        data (list): List of devices found, in order
        names (list): Device names, in order found
        uuids (list): UUID
        by_name (dict): Access to each device by devicename
        by_uuid (dict): Access to each device by uuid
        info (list): List of devices found, in order using SetupInfo structure

    Example:
        >>> len(setup_info)
        6
        >>> setup_info.names[0]
        'VG00-tmp'
        >>> setup_info[1]['Maj']
        '253'
        >>> setup_info[1]['Stat']
        'L--w'
        >>> setup_info.info[-1].name
        'VG00-var_log_audit'
        >>> setup_info.info[-1].major
        253
        >>> setup_info.info[-1].live_table
        True
        >>> setup_info.info[-1].readonly
        False
    """

    def parse_content(self, content):
        self.data = parse_delimited_table(content)

        self.names = [dm['Name'] for dm in self.data if 'Name' in dm]
        self.by_name = dict((dm['Name'], dm) for dm in self.data if 'Name' in dm)
        self.uuids = [dm['UUID'] for dm in self.data if 'UUID' in dm]
        self.by_uuid = dict((dm['UUID'], dm) for dm in self.data if 'UUID' in dm)
        self.info = []
        for dm in self.data:
            self.info.append(SetupInfo(
                name=dm.get('Name'),
                major=int(dm.get('Maj')) if 'Maj' in dm and dm.get('Maj').isdigit() else None,
                minor=int(dm.get('Min')) if 'Min' in dm and dm.get('Min').isdigit() else None,
                open=int(dm.get('Open')) if 'Open' in dm and dm.get('Open').isdigit() else None,
                segments=int(dm.get('Targ')) if 'Targ' in dm and dm.get('Targ').isdigit() else None,
                events=int(dm.get('Event')) if 'Event' in dm and dm.get('Event').isdigit() else None,
                live_table=dm.get('Stat', '----')[0] == 'L',
                inactive_table=dm.get('Stat', '----')[1] == 'I',
                suspended=dm.get('Stat', '----')[2] == 's',
                readonly=dm.get('Stat', '----')[3] == 'r',
                uuid=dm.get('UUID')
            ))

    def __len__(self):
        """
        The length of the devices list
        """
        return len(self.data)

    def __iter__(self):
        """
        Iterate through the devices list
        """
        for dm in self.data:
            yield dm

    def __getitem__(self, idx):
        """
        Fetch a device by index in devices list
        """
        return self.data[idx]



"""
NeutronMl2Conf - file ``/etc/neutron/plugins/ml2/ml2_conf.ini``
===============================================================
This class provides parsing for the file ``/etc/neutron/plugins/ml2/ml2_conf.ini``.
See the ``IniConfigFile`` class for more usage information.
"""


from .. import IniConfigFile, parser, add_filter
from insights.specs import Specs

add_filter(Specs.neutron_ml2_conf, ["["])


@parser(Specs.neutron_ml2_conf)
class NeutronMl2Conf(IniConfigFile):
    """
    Neutron ML2 configuration parser class, based on the ``IniConfigFile`` class.

    Sample input data is in the format::

        [DEFAULT]

        debug = false
        verbose = true
        use_syslog = false

        [ml2]

        type_drivers = local,flat,vlan,gre,vxlan,geneve
        mechanism_drivers = openvswitch

        [securitygroup]

        firewall_driver = iptables_hybrid
        enable_security_group = true


    Examples:

        >>> neutron_ml2_conf.has_option("ml2", "type_drivers")
        True
        >>> neutron_ml2_conf.get("ml2", "mechanism_drivers") == "openvswitch"
        True

    """
    pass



"""
GFS2FileSystemBlockSize - command ``stat -fc %s <mount_point_path>``
====================================================================

The parser parse the output of ``stat -fc %s <mount_point_path>``
"""

from insights import parser, CommandParser
from insights.specs import Specs
from insights.parsers import SkipException


@parser(Specs.gfs2_file_system_block_size)
class GFS2FileSystemBlockSize(CommandParser):
    """
    Class for parsing ``stat -fc %s <mount_point_path>`` command output.
    The size is kept in the ``block_size`` property.
    Typical output of command ``stat -fc %s <mount_point_path>`` looks like::

        4096

    Examples::

        >>> type(gfs2_mp)
        <class 'insights.parsers.gfs2_file_system_block_size.GFS2FileSystemBlockSize'>
        >>> gfs2_mp.block_size
        4096

    Raise::

        SkipException: When the content isn't in the expected format.

    Attributes::

        block_size (int): The block size of the gfs2 file system.
    """

    def parse_content(self, content):
        if len(content) == 1 and content[0].isdigit():
            self.block_size = int(content[0])
        else:
            raise SkipException('The output is invalid.')



import doctest
import pytest
from insights.parsers import avc_cache_threshold, ParseException
from insights.parsers.avc_cache_threshold import AvcCacheThreshold
from insights.tests import context_wrap

AVC_CACHE_THRESHOLD = """
512
""".strip()

AVC_CACHE_THRESHOLD_INVALID = """
invalid
invalid
invalid
""".strip()


def test_avc_cache_threshold():
    cache_threshold = avc_cache_threshold.AvcCacheThreshold(context_wrap(AVC_CACHE_THRESHOLD))
    assert cache_threshold.cache_threshold == 512


def test_invalid():
    with pytest.raises(ParseException) as e:
        avc_cache_threshold.AvcCacheThreshold(context_wrap(AVC_CACHE_THRESHOLD_INVALID))
    assert "invalid" in str(e)


def test_avc_cache_threshold_doc_examples():
    env = {
        'avc_cache_threshold': AvcCacheThreshold(
            context_wrap(AVC_CACHE_THRESHOLD)),
    }
    failed, total = doctest.testmod(avc_cache_threshold, globs=env)
    assert failed == 0



"""
MariaDBLog - File ``/var/log/mariadb/mariadb.log``
==================================================

Module for parsing the log file for MariaDB

Typical content of ``mariadb.log`` file is::

    161109  9:25:42 [Note] WSREP: Read nil XID from storage engines, skipping position init
    161109  9:25:42 [Note] WSREP: wsrep_load(): loading provider library 'none'
    161109  9:25:42 [Warning] Failed to setup SSL
    161109  9:25:42 [Warning] SSL error: SSL_CTX_set_default_verify_paths failed
    161109  9:25:42 [Note] WSREP: Service disconnected.
    161109  9:25:43 [Note] WSREP: Some threads may fail to exit.
    161109  9:25:43 [Note] WSREP: Read nil XID from storage engines, skipping position init
    161109  9:25:43 [Note] WSREP: wsrep_load(): loading provider library 'none'
    161109  9:25:43 [Warning] Failed to setup SSL
    161109  9:25:43 [Warning] SSL error: SSL_CTX_set_default_verify_paths failed
    161109  9:25:43 [Note] WSREP: Service disconnected.
    161109  9:25:44 [Note] WSREP: Some threads may fail to exit.
    161109 14:28:22 mysqld_safe Starting mysqld daemon with databases from /var/lib/mysql
    161109 14:28:22 mysqld_safe WSREP: Running position recovery with --log_error='/var/lib/mysql/wsrep_recovery.OkURTZ' --pid-file='/var/lib/mysql/overcloud-controller-0.localdomain-recover.pid'
    161109 14:28:22 [Warning] option 'open_files_limit': unsigned value 18446744073709551615 adjusted to 4294967295

Examples:

    >>> mdb = shared[MariaDBLog]
    >>> mdb.get('mysqld_safe')[0]['raw_message']
    '161109 14:28:22 mysqld_safe Starting mysqld daemon with databases from /var/lib/mysql'
    >>> 'SSL_CTX_set_default_verify_paths' in mdb
    True
"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.mariadb_log)
class MariaDBLog(LogFileOutput):
    """Class for parsing ``/var/log/mariadb/mariadb.log`` file.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`
    """
    time_format = {
        'pre_10.1.5': '%y%m%d %H:%M:%S',
        'post_10.1.5': '%Y-%m-%d %H:%M:%S'
    }



"""
VMwareToolsConf - file ``/etc/vmware-tools/tools.conf``
=======================================================

The VMware tools configuration file ``/etc/vmware-tools/tools.conf``
is in the standard 'ini' format and is read by the IniConfigFile
parser. ``vmtoolsd.service`` provided by ``open-vm-tools`` package is
configured using ``/etc/vmware-tools/tools.conf``.


Sample ``/etc/vmware-tools/tools.conf`` file::

    [guestinfo]
    disable-query-diskinfo = true

    [logging]
    log = true

    vmtoolsd.level = debug
    vmtoolsd.handler = file
    vmtoolsd.data = /tmp/vmtoolsd.log


Examples:

    >>> list(conf.sections()) == [u'guestinfo', u'logging']
    True
    >>> conf.has_option('guestinfo', 'disable-query-diskinfo')
    True
    >>> conf.getboolean('guestinfo', 'disable-query-diskinfo')
    True
    >>> conf.get('guestinfo', 'disable-query-diskinfo') == u'true'
    True

"""
from .. import IniConfigFile, parser
from insights.specs import Specs


@parser(Specs.vmware_tools_conf)
class VMwareToolsConf(IniConfigFile):
    """Class for VMware tool configuration file content."""
    pass



import pytest
import doctest
from insights.parsers import aws_instance_type
from insights.parsers.aws_instance_type import AWSInstanceType
from insights.tests import context_wrap
from insights.parsers import SkipException, ParseException
from insights.core.plugins import ContentException

AWS_TYPE = "r3.xlarge"
AWS_TYPE_CURL_STATS = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed

   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k
r3.xlarge"""
AWS_TYPE_AB_1 = """
curl: (7) Failed to connect to 169.254.169.254 port 80: Connection timed out
""".strip()
AWS_TYPE_AB_2 = """
curl: (7) couldn't connect to host
""".strip()
AWS_TYPE_AB_3 = """
curl: (28) connect() timed out!
""".strip()
AWS_TYPE_AB_4 = """
.micro
""".strip()
AWS_TYPE_AB_5 = """
No module named insights.tools
""".strip()


def test_aws_instance_type_ab_other():
    with pytest.raises(SkipException):
        AWSInstanceType(context_wrap(AWS_TYPE_AB_1))

    with pytest.raises(SkipException):
        AWSInstanceType(context_wrap(AWS_TYPE_AB_2))

    with pytest.raises(SkipException):
        AWSInstanceType(context_wrap(AWS_TYPE_AB_3))

    with pytest.raises(ParseException) as pe:
        AWSInstanceType(context_wrap(AWS_TYPE_AB_4))
        assert 'Unrecognized type' in str(pe)

    with pytest.raises(ContentException) as pe:
        AWSInstanceType(context_wrap(AWS_TYPE_AB_5))


def test_aws_instance_type_ab_empty():
    with pytest.raises(SkipException):
        AWSInstanceType(context_wrap(''))


def test_aws_instance_type():
    aws = AWSInstanceType(context_wrap(AWS_TYPE))
    assert aws.type == "R3"
    assert aws.raw == "r3.xlarge"
    assert 'large' in str(aws)


def test_aws_instance_type_stats():
    aws = AWSInstanceType(context_wrap(AWS_TYPE_CURL_STATS))
    assert aws.type == "R3"
    assert aws.raw == "r3.xlarge"
    assert 'large' in str(aws)


def test_doc_examples():
    env = {
            'aws_inst': AWSInstanceType(context_wrap(AWS_TYPE))
          }
    failed, total = doctest.testmod(aws_instance_type, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.parsers import (
    puppet_ca_cert_expire_date, SkipException, ParseException)
from insights.tests import context_wrap


PUPPET_CERT_EXPIRE_INFO = '''
notAfter=Dec  4 07:04:05 2035 GMT
'''

WRONG_PUPPET_CERT_INFO_1 = '''
Can't open /etc/puppetlabs/puppet/ssl/ca/ca_crt.pem for reading, No such file or directory
140033749546816:error:02001002:system library:fopen:No such file or directory:crypto/bio/bss_file.c:69:fopen('/etc/puppetlabs/puppet/ssl/ca/ca_crt.pem','r')
140033749546816:error:2006D080:BIO routines:BIO_new_file:no such file:crypto/bio/bss_file.c:76:
unable to load certificate
'''

WRONG_PUPPET_CERT_INFO_2 = '''
notAfter=Mon Jan  4 02:31:28 EST 202
'''

WRONG_PUPPET_CERT_INFO_3 = '''
Mon Jan  4 02:31:28 EST 202
'''


def test_HTL_doc_examples():
    date_info = puppet_ca_cert_expire_date.PuppetCertExpireDate(context_wrap(PUPPET_CERT_EXPIRE_INFO))
    globs = {
        'date_info': date_info
    }
    failed, tested = doctest.testmod(puppet_ca_cert_expire_date, globs=globs)
    assert failed == 0


def test_wrong_output():
    with pytest.raises(SkipException):
        puppet_ca_cert_expire_date.PuppetCertExpireDate(context_wrap(WRONG_PUPPET_CERT_INFO_1))
    with pytest.raises(SkipException):
        puppet_ca_cert_expire_date.PuppetCertExpireDate(context_wrap(WRONG_PUPPET_CERT_INFO_3))
    with pytest.raises(ParseException):
        puppet_ca_cert_expire_date.PuppetCertExpireDate(context_wrap(WRONG_PUPPET_CERT_INFO_2))



from ...parsers.chkconfig import ChkConfig
from ...parsers.systemd.unitfiles import UnitFiles
from ..services import Services
from ...tests import context_wrap

CHKCONIFG = """
auditd         	0:off	1:off	2:on	3:on	4:on	5:on	6:off
crond          	0:off	1:off	2:on	3:on	4:on	5:on	6:off
iptables       	0:off	1:off	2:on	3:on	4:on	5:on	6:off
kdump          	0:off	1:off	2:off	3:on	4:on	5:on	6:off
restorecond    	0:off	1:off	2:off	3:off	4:off	5:off	6:off
""".strip()

# Worth noting that actual systemctl list-unit-files output has spaces on
# the ends, but we strip them.
SYSTEMCTL = """
UNIT FILE                                   STATE
auditd.service                              enabled
cpupower.service                            disabled
crond.service                               enabled
emergency.service                           static
firewalld.service                           enabled
fstrim.service                              static
kdump.service                               enabled

7 unit files listed.
""".strip()


def test_chkconfig():
    context = context_wrap(CHKCONIFG)
    chkconfig = ChkConfig(context)
    services = Services(chkconfig, None)
    assert len(services.services) == 5
    assert len(services.parsed_lines) == 5
    assert services.is_on('auditd')
    assert services.is_on('crond')
    assert not services.is_on('restorecond')
    assert not services.is_on('ksm')
    assert services.service_line('auditd') == "auditd         	0:off	1:off	2:on	3:on	4:on	5:on	6:off"
    assert services.service_line('ksm') == ''
    # Test __contains__ functionality
    assert 'crond' in services
    assert 'ksm' not in services


def test_systemctl():
    context = context_wrap(SYSTEMCTL)
    unitfiles = UnitFiles(context)
    services = Services(None, unitfiles)
    assert len(services.services) == 7
    assert len(services.parsed_lines) == 7
    assert services.is_on('auditd.service')
    assert services.is_on('crond.service')
    assert not services.is_on('cpupower.service')
    assert services.is_on('auditd')
    assert services.is_on('crond')
    assert not services.is_on('cpupower')
    assert not services.is_on('ksm')
    assert not services.is_on('ksm.service')
    assert services.service_line('auditd.service') == "auditd.service                              enabled"
    # Test __contains__ functionality
    assert 'crond' in services
    assert 'ksm' not in services


def test_combined():
    context = context_wrap(CHKCONIFG)
    chkconfig = ChkConfig(context)
    context = context_wrap(SYSTEMCTL)
    unitfiles = UnitFiles(context)
    services = Services(chkconfig, unitfiles)
    assert len(services.services) == 12
    assert len(services.parsed_lines) == 12
    assert services.is_on('auditd')
    assert services.is_on('crond')
    assert not services.is_on('restorecond')
    assert services.is_on('auditd.service')
    assert services.is_on('crond.service')
    assert not services.is_on('cpupower.service')
    assert not services.is_on('cpupower')
    # Test __contains__ functionality
    assert 'crond' in services
    assert 'ksm' not in services



from insights.parsers import openvswitch_logs
from insights.tests import context_wrap

from datetime import datetime

LOG_LINES = """
2017-03-12T11:56:54.989Z|00001|vlog|INFO|opened log file /var/log/openvswitch/ovsdb-server.log
2017-03-12T11:56:55.053Z|00002|ovsdb_server|INFO|ovsdb-server (Open vSwitch) 2.4.0
2017-03-12T11:57:04.988Z|00003|memory|INFO|2664 kB peak resident set size after 10.0 seconds
2017-03-12T11:57:04.988Z|00004|memory|INFO|cells:816 monitors:1 sessions:2
2017-03-13T07:41:01.524Z|00005|vlog|INFO|closing log file
2017-03-13T07:41:02.524Z|00005|testing|INFO|testing handing of messages with | in them
2017-03-13T07:41:03.524Z|testing messages with not enough fields in them.
"""


def test_ovsdb_server():
    logs = openvswitch_logs.OVSDB_Server_Log(context_wrap(LOG_LINES))

    assert len(logs) == 7
    vlogs = logs.get('vlog')
    assert len(vlogs) == 2
    assert vlogs[0] == {
        'raw_message': '2017-03-12T11:56:54.989Z|00001|vlog|INFO|opened log file /var/log/openvswitch/ovsdb-server.log',
        'timestamp': '2017-03-12T11:56:54.989Z',
        'sequence': '00001',
        'module': 'vlog',
        'level': 'INFO',
        'message': 'opened log file /var/log/openvswitch/ovsdb-server.log'
    }
    assert vlogs[1] == {
        'raw_message': '2017-03-13T07:41:01.524Z|00005|vlog|INFO|closing log file',
        'timestamp': '2017-03-13T07:41:01.524Z',
        'sequence': '00005',
        'module': 'vlog',
        'level': 'INFO',
        'message': 'closing log file'
    }
    assert len(list(logs.get_after(datetime(2017, 3, 13, 0, 0, 0)))) == 3
    test_logs = logs.get('testing')
    assert test_logs[0] == {
        'raw_message': '2017-03-13T07:41:02.524Z|00005|testing|INFO|testing handing of messages with | in them',
        'timestamp': '2017-03-13T07:41:02.524Z',
        'sequence': '00005',
        'module': 'testing',
        'level': 'INFO',
        'message': 'testing handing of messages with | in them',
    }
    assert test_logs[1] == {
        'raw_message': '2017-03-13T07:41:03.524Z|testing messages with not enough fields in them.'
    }
    assert len(list(logs.get_after(datetime(2017, 3, 12, 11, 20, 0), 'testing'))) == 2


# Same tests, same file, just a different parser name.
def test_ovs_vswitch():
    logs = openvswitch_logs.OVS_VSwitchd_Log(context_wrap(LOG_LINES))

    assert len(logs) == 7
    vlogs = logs.get('vlog')
    assert len(vlogs) == 2
    assert vlogs[0] == {
        'raw_message': '2017-03-12T11:56:54.989Z|00001|vlog|INFO|opened log file /var/log/openvswitch/ovsdb-server.log',
        'timestamp': '2017-03-12T11:56:54.989Z',
        'sequence': '00001',
        'module': 'vlog',
        'level': 'INFO',
        'message': 'opened log file /var/log/openvswitch/ovsdb-server.log'
    }
    assert vlogs[1] == {
        'raw_message': '2017-03-13T07:41:01.524Z|00005|vlog|INFO|closing log file',
        'timestamp': '2017-03-13T07:41:01.524Z',
        'sequence': '00005',
        'module': 'vlog',
        'level': 'INFO',
        'message': 'closing log file'
    }
    assert len(list(logs.get_after(datetime(2017, 3, 13, 0, 0, 0)))) == 3
    assert len(list(logs.get_after(datetime(2017, 3, 12, 11, 20, 0), 'testing'))) == 2



from insights.parsers.keystone import KeystoneConf
from insights.tests import context_wrap

KEYSTONE_CONF = """
[DEFAULT]

#
# From keystone
#
admin_token = ADMIN
compute_port = 8774

[identity]

# From keystone
default_domain_id = default
#domain_specific_drivers_enabled = false
domain_configurations_from_database = false

[identity_mapping]

driver = keystone.identity.mapping_backends.sql.Mapping
generator = keystone.identity.id_generators.sha256.Generator
#backward_compatible_ids = true
""".strip()


def test_keystone():
    kconf = KeystoneConf(context_wrap(KEYSTONE_CONF))
    assert kconf is not None
    assert kconf.defaults() == {'admin_token': 'ADMIN',
                                'compute_port': '8774'}
    assert 'identity' in kconf
    assert 'identity_mapping' in kconf
    assert kconf.has_option('identity', 'default_domain_id')
    assert kconf.has_option('identity_mapping', 'driver')
    assert not kconf.has_option('identity', 'domain_specific_drivers_enabled')
    assert kconf.get('identity', 'default_domain_id') == 'default'
    assert kconf.items('DEFAULT') == {'admin_token': 'ADMIN',
                                      'compute_port': '8774'}



from insights.parsers import vgdisplay
from insights.tests import context_wrap


VGDISPLAY = """
Couldn't find device with uuid VVLmw8-e2AA-ECfW-wDPl-Vnaa-0wW1-utv7tV.
  There are 1 physical volumes missing.
    Couldn't find device with uuid VVLmw8-e2AA-ECfW-wDPl-Vnaa-0wW1-utv7tV.
  There are 1 physical volumes missing.
  --- Volume group ---
  VG Name               rhel_hp-dl160g8-3
  Format                lvm2
  System ID
  Metadata Areas        1
  Metadata Sequence No  5
  VG Access             read/write
  Total PE              119109
  Alloc PE / Size       119098 / 465.23 GiB
  Free  PE / Size       11 / 44.00 MiB
  VG UUID               by0Dl3-0lpB-MxEz-f6GO-9LYO-YRAQ-GufNZD

  VG Name               rhel_hp-dl260g7-4
  Format                lvm2
  System ID
  VG Access             read/write
  Alloc PE / Size       119098 / 465.23 GiB
  Free  PE / Size       11 / 44.00 MiB
  VG UUID               by0Dl3-0lpB-MxEz-f6GO-9LYO-YRAQ-GufNZN
""".strip()

VGDISPLAY_VV = """
      Setting activation/monitoring to 1
      Setting global/locking_type to 1
      Setting global/wait_for_locks to 1
      File-based locking selected.
      Setting global/prioritise_write_locks to 1
      Setting global/locking_dir to /run/lock/lvm
      Setting global/use_lvmlockd to 0
      Setting response to OK
      Setting token to filter:3239235440
      Setting daemon_pid to 856
      Setting response to OK
      Setting global_disable to 0
      Setting response to OK
      Setting response to OK
      Setting response to OK
      Setting name to RHEL7CSB
      report/output_format not found in config: defaulting to basic
      log/report_command_log not found in config: defaulting to 0
      Processing VG RHEL7CSB aeMrAJ-QkAe-llvW-oAoE-CWLF-MnUd-edD1tI
      Locking /run/lock/lvm/V_RHEL7CSB RB
      Reading VG RHEL7CSB aeMrAJQkAellvWoAoECWLFMnUdedD1tI
      Setting response to OK
      Setting response to OK
      Setting response to OK
      Setting name to RHEL7CSB
      Setting metadata/format to lvm2
      Setting id to EfWV9V-03CX-E6zc-JkMw-yQae-wdzp-Je1KUn
      Setting format to lvm2
      Setting device to 64768
      Setting dev_size to 970475520
      Setting label_sector to 1
      Setting ext_flags to 0
      Setting ext_version to 1
      Setting size to 1044480
      Setting start to 4096
      Setting ignore to 0
      Setting response to OK
      Setting response to OK
      Setting response to OK
      /dev/mapper/luks-96c66446-77fd-4431-9508-f6912bd84194: size is 970475520 sectors
      Process single VG RHEL7CSB
  --- Volume group ---
  VG Name               RHEL7CSB
  System ID
  Format                lvm2
  Metadata Areas        1
  Metadata Sequence No  13
  VG Access             read/write
  VG Status             resizable
  MAX LV                0
  Cur LV                7
  Open LV               6
  Max PV                0
  Cur PV                2
  Act PV                1
  VG Size               462.76 GiB
  PE Size               4.00 MiB
  Total PE              118466
  Alloc PE / Size       114430 / 446.99 GiB
  Free  PE / Size       4036 / 15.77 GiB
  VG UUID               aeMrAJ-QkAe-llvW-oAoE-CWLF-MnUd-edD1tI

      Adding RHEL7CSB/Home to the list of LVs to be processed.
      Adding RHEL7CSB/Root to the list of LVs to be processed.
      Adding RHEL7CSB/Swap to the list of LVs to be processed.
      Adding RHEL7CSB/VMs_lv to the list of LVs to be processed.
      Adding RHEL7CSB/NotBackedUp_lv to the list of LVs to be processed.
      Adding RHEL7CSB/ISOs_lv to the list of LVs to be processed.
      Adding RHEL7CSB/RHEL6-pg-pgsql-lv to the list of LVs to be processed.
      Processing LV Home in VG RHEL7CSB.
  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/RHEL7CSB/Home
  LV Name                Home
  VG Name                RHEL7CSB
  LV UUID                IdRMoU-JorV-ChPg-F1zb-6np9-yc08-qxj08f
  LV Write Access        read/write
  LV Creation host, time localhost, 2015-04-16 00:02:47 +1000
  LV Status              available
  # open                 1
  LV Size                100.00 GiB
  Current LE             25600
  Segments               1
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:3

      Processing LV Root in VG RHEL7CSB.
  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/RHEL7CSB/Root
  LV Name                Root
  VG Name                RHEL7CSB
  LV UUID                lXBbNv-u1r6-qCo1-682K-w5hW-ED8A-Sl4gvf
  LV Write Access        read/write
  LV Creation host, time localhost, 2015-04-16 00:02:50 +1000
  LV Status              available
  # open                 1
  LV Size                29.30 GiB
  Current LE             7500
  Segments               1
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:1

      Processing LV Swap in VG RHEL7CSB.
  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/RHEL7CSB/Swap
  LV Name                Swap
  VG Name                RHEL7CSB
  LV UUID                R2ErFM-Rrql-L0tH-VIbm-F0Km-P7uW-4hk3rl
  LV Write Access        read/write
  LV Creation host, time localhost, 2015-04-15 14:02:52 +1000
  LV Status              available
  # open                 2
  LV Size                7.70 GiB
  Current LE             1970
  Segments               1
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:2

      Processing LV VMs_lv in VG RHEL7CSB.
  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/RHEL7CSB/VMs_lv
  LV Name                VMs_lv
  VG Name                RHEL7CSB
  LV UUID                iXOy1p-wczA-WEEy-mawN-EPOh-YZGy-KIpTls
  LV Write Access        read/write
  LV Creation host, time localhost.localdomain, 2015-04-15 15:52:53 +1000
  LV Status              available
  # open                 1
  LV Size                120.00 GiB
  Current LE             30720
  Segments               2
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:4

      Processing LV NotBackedUp_lv in VG RHEL7CSB.
  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/RHEL7CSB/NotBackedUp_lv
  LV Name                NotBackedUp_lv
  VG Name                RHEL7CSB
  LV UUID                8SI5e4-O5uA-TbNC-bZeY-1WPg-Zf3P-H63Xsi
  LV Write Access        read/write
  LV Creation host, time pwayper.remote.csb, 2015-04-15 16:30:00 +1000
  LV Status              available
  # open                 1
  LV Size                100.00 GiB
  Current LE             25600
  Segments               1
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:5

      Processing LV ISOs_lv in VG RHEL7CSB.
  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/RHEL7CSB/ISOs_lv
  LV Name                ISOs_lv
  VG Name                RHEL7CSB
  LV UUID                YVI2nw-7LOu-mseA-vQkC-HpcK-Xabx-WK23yM
  LV Write Access        read/write
  LV Creation host, time pwayper.remote.csb, 2015-04-15 16:30:52 +1000
  LV Status              available
  # open                 1
  LV Size                50.00 GiB
  Current LE             12800
  Segments               1
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:6

      Processing LV RHEL6-pg-pgsql-lv in VG RHEL7CSB.
  --- Logical volume ---
      global/lvdisplay_shows_full_device_path not found in config: defaulting to 0
  LV Path                /dev/RHEL7CSB/RHEL6-pg-pgsql-lv
  LV Name                RHEL6-pg-pgsql-lv
  VG Name                RHEL7CSB
  LV UUID                USkJVW-ALIP-kcpt-Av5e-Vf2u-GqXr-1VXhb1
  LV Write Access        read/write
  LV Creation host, time pwayper.remote.csb, 2015-04-19 15:36:34 +1000
  LV Status              available
  # open                 0
  LV Size                40.00 GiB
  Current LE             10240
  Segments               1
  Allocation             inherit
  Read ahead sectors     auto
  - currently set to     256
  Block device           253:7

  --- Physical volumes ---
  PV Name               /dev/mapper/luks-96c66446-77fd-4431-9508-f6912bd84194
  PV UUID               EfWV9V-03CX-E6zc-JkMw-yQae-wdzp-Je1KUn
  PV Status             allocatable
  Total PE / Free PE    118466 / 4036

  PV Name               /dev/sde
  PV UUID               bh4MbE-USrx-6Xd0-3biH-8v5o-Ztzn-XvKUkX
  PV Status             allocatable
  Total PE / Free PE    715396 / 212932

      Unlocking /run/lock/lvm/V_RHEL7CSB
      Setting global/notify_dbus to 1
"""


class TestVGdisplay():
    def test_VgDisplay(self):
        vg_info = vgdisplay.VgDisplay(context_wrap(VGDISPLAY))
        assert hasattr(vg_info, 'vg_list')
        assert vg_info.vg_list[0].get('VG Name') == 'rhel_hp-dl160g8-3'
        assert vg_info.vg_list[0].get('Metadata Sequence No') == '5'
        assert vg_info.vg_list[1].get('VG UUID') == 'by0Dl3-0lpB-MxEz-f6GO-9LYO-YRAQ-GufNZN'
        assert hasattr(vg_info, 'debug_info')
        assert vg_info.debug_info[0] == "Couldn't find device with uuid VVLmw8-e2AA-ECfW-wDPl-Vnaa-0wW1-utv7tV."
        assert vg_info.debug_info[1] == "There are 1 physical volumes missing."

    def test_vgdisplay_vv(self):
        vg_info = vgdisplay.VgDisplay(context_wrap(VGDISPLAY_VV))

        # Volume group tests
        assert hasattr(vg_info, 'vg_list')
        vgdata = vg_info.vg_list
        assert isinstance(vgdata, list)
        assert len(vgdata) == 1
        vg = vgdata[0]
        assert isinstance(vg, dict)
        assert sorted(vg.keys()) == sorted([
            'VG Name', 'Format', 'Metadata Areas', 'Metadata Sequence No',
            'VG Access', 'VG Status', 'MAX LV', 'Cur LV', 'Open LV',
            'Max PV', 'Cur PV', 'Act PV', 'VG Size', 'PE Size', 'Total PE',
            'Alloc PE / Size', 'Free  PE / Size', 'VG UUID',
            'Logical Volumes', 'Physical Volumes'
        ])
        assert vg['VG Name'] == 'RHEL7CSB'
        assert vg['Format'] == 'lvm2'
        assert vg['Metadata Areas'] == '1'
        assert vg['Metadata Sequence No'] == '13'
        assert vg['VG Access'] == 'read/write'
        assert vg['VG Status'] == 'resizable'
        assert vg['MAX LV'] == '0'
        assert vg['Cur LV'] == '7'
        assert vg['Open LV'] == '6'
        assert vg['Max PV'] == '0'
        assert vg['Cur PV'] == '2'
        assert vg['Act PV'] == '1'
        assert vg['VG Size'] == '462.76 GiB'
        assert vg['PE Size'] == '4.00 MiB'
        assert vg['Total PE'] == '118466'
        assert vg['Alloc PE / Size'] == '114430 / 446.99 GiB'
        # Caveat Utilor: double space after Free for alignment purposes
        assert vg['Free  PE / Size'] == '4036 / 15.77 GiB'
        assert vg['VG UUID'] == 'aeMrAJ-QkAe-llvW-oAoE-CWLF-MnUd-edD1tI'

        # Logical volume tests
        assert isinstance(vg['Logical Volumes'], dict)
        assert len(vg['Logical Volumes']) == 7
        # Test the first LV and then get a few different keys
        assert sorted(vg['Logical Volumes'].keys()) == sorted([
            '/dev/RHEL7CSB/Home', '/dev/RHEL7CSB/ISOs_lv',
            '/dev/RHEL7CSB/NotBackedUp_lv', '/dev/RHEL7CSB/RHEL6-pg-pgsql-lv',
            '/dev/RHEL7CSB/Root', '/dev/RHEL7CSB/Swap', '/dev/RHEL7CSB/VMs_lv'
        ])
        lvhome = vg['Logical Volumes']['/dev/RHEL7CSB/Home']
        assert isinstance(lvhome, dict)
        assert sorted(lvhome.keys()) == sorted([
            'LV Path', 'LV Name', 'VG Name', 'LV UUID', 'LV Write Access',
            'LV Creation host, time', 'LV Status', '# open', 'LV Size',
            'Current LE', 'Segments', 'Allocation', 'Read ahead sectors',
            '- currently set to', 'Block device'
        ])
        assert lvhome['LV Path'] == '/dev/RHEL7CSB/Home'
        assert lvhome['LV Name'] == 'Home'
        assert lvhome['VG Name'] == 'RHEL7CSB'
        assert lvhome['LV UUID'] == 'IdRMoU-JorV-ChPg-F1zb-6np9-yc08-qxj08f'
        assert lvhome['LV Write Access'] == 'read/write'
        assert lvhome['LV Creation host, time'] == 'localhost, 2015-04-16 00:02:47 +1000'
        assert lvhome['LV Status'] == 'available'
        assert lvhome['# open'] == '1'
        assert lvhome['LV Size'] == '100.00 GiB'
        assert lvhome['Current LE'] == '25600'
        assert lvhome['Segments'] == '1'
        assert lvhome['Allocation'] == 'inherit'
        assert lvhome['Read ahead sectors'] == 'auto'
        assert lvhome['- currently set to'] == '256'
        assert lvhome['Block device'] == '253:3'

        # Physical volume tests
        assert isinstance(vg['Physical Volumes'], dict)
        assert len(vg['Physical Volumes']) == 2
        assert '/dev/mapper/luks-96c66446-77fd-4431-9508-f6912bd84194' in vg['Physical Volumes']
        pvluks = vg['Physical Volumes']['/dev/mapper/luks-96c66446-77fd-4431-9508-f6912bd84194']
        assert sorted(pvluks.keys()) == sorted([
            'PV Name', 'PV UUID', 'PV Status', 'Total PE / Free PE'
        ])
        assert pvluks['PV Name'] == '/dev/mapper/luks-96c66446-77fd-4431-9508-f6912bd84194'
        assert pvluks['PV UUID'] == 'EfWV9V-03CX-E6zc-JkMw-yQae-wdzp-Je1KUn'
        assert pvluks['PV Status'] == 'allocatable'
        assert pvluks['Total PE / Free PE'] == '118466 / 4036'

        # Debug info tests
        assert hasattr(vg_info, 'debug_info')
        assert isinstance(vg_info.debug_info, list)
        assert vg_info.debug_info == []



"""
Domain information groper (Dig) parsers
=======================================

Parsers included in this module are:

DigDnssec - command ``/usr/bin/dig +dnssec . SOA``
--------------------------------------------------
DigEdns - command ``/usr/bin/dig +edns=0 . SOA``
------------------------------------------------
DigNoedns - command ``/usr/bin/dig +noedns . SOA``
--------------------------------------------------
"""

import re

from insights.core import CommandParser
from insights.core.plugins import parser
from insights.parsers import SkipException
from insights.specs import Specs


HEADER_TEMPLATE = re.compile(r';; ->>HEADER<<-.*status: (\S+),')
RRSIG_TEMPLATE = re.compile(r'RRSIG')


class Dig(CommandParser):
    """
    Base class for classes using ``dig`` command.

    Attributes:
        status (string): Determines if the lookup succeeded.
        has_signature (bool): True, if signature is present.
        command (string): Specific ``dig`` command used.

    Raises:
        SkipException: When content is empty or cannot be parsed.
    """
    def __init__(self, context, command):
        self.status = None
        self.has_signature = False
        self.command = command
        super(Dig, self).__init__(context)

    def parse_content(self, content):
        if not content:
            raise SkipException('No content.')

        for line in content:
            match = HEADER_TEMPLATE.search(line)
            if match:
                self.status = match.group(1)
            if RRSIG_TEMPLATE.search(line):
                self.has_signature = True


@parser(Specs.dig_dnssec)
class DigDnssec(Dig):
    """
    Class for parsing ``/usr/bin/dig +dnssec . SOA`` command.

    Sample output of this command is::

        ; <<>> DiG 9.11.1-P3-RedHat-9.11.1-2.P3.fc26 <<>> +dnssec nic.cz. SOA
        ;; global options: +cmd
        ;; Got answer:
        ;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 58794
        ;; flags: qr rd ra; QUERY: 1, ANSWER: 2, AUTHORITY: 0, ADDITIONAL: 1

        ;; OPT PSEUDOSECTION:
        ; EDNS: version: 0, flags: do; udp: 4096
        ;; QUESTION SECTION:
        ;nic.cz.                                IN      SOA

        ;; ANSWER SECTION:
        nic.cz.                 278     IN      SOA     a.ns.nic.cz.
        hostmaster.nic.cz. 1508686803 10800 3600 1209600 7200
        nic.cz.                 278     IN      RRSIG   SOA 13 2 1800
        20171105143612 20171022144003 41758 nic.cz.
        hq3rr8dASRlucMJxu2QZnX6MVaMYsKhmGGxBOwpkeUrGjfo6clzG6MZN
        2Jy78fWYC/uwyIsI3nZMUKv573eCWg==

        ;; Query time: 22 msec
        ;; SERVER: 10.38.5.26#53(10.38.5.26)
        ;; WHEN: Tue Oct 24 14:28:56 CEST 2017
        ;; MSG SIZE  rcvd: 189

    Examples:
        >>> dig_dnssec.status
        'NOERROR'
        >>> dig_dnssec.has_signature
        True
        >>> dig_dnssec.command
        '/usr/bin/dig +dnssec . SOA'
    """
    def __init__(self, context):
        super(DigDnssec, self).__init__(context, '/usr/bin/dig +dnssec . SOA')


@parser(Specs.dig_edns)
class DigEdns(Dig):
    """
    Class for parsing ``/usr/bin/dig +edns=0 . SOA`` command.

    Sample output of this command is::

        ; <<>> DiG 9.11.1-P3-RedHat-9.11.1-3.P3.fc26 <<>> +edns=0 . SOA
        ;; global options: +cmd
        ;; Got answer:
        ;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 11158
        ;; flags: qr rd ra ad; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 1

        ;; OPT PSEUDOSECTION:
        ; EDNS: version: 0, flags:; udp: 4096
        ;; QUESTION SECTION:
        ;.				IN	SOA

        ;; ANSWER SECTION:
        .			19766	IN	SOA	a.root-servers.net. nstld.verisign-grs.com. 2017120600 1800 900 604800 86400

        ;; Query time: 22 msec
        ;; SERVER: 10.38.5.26#53(10.38.5.26)
        ;; WHEN: Thu Dec 07 09:38:33 CET 2017
        ;; MSG SIZE  rcvd: 103

    Examples:
        >>> dig_edns.status
        'NOERROR'
        >>> dig_edns.has_signature
        False
        >>> dig_edns.command
        '/usr/bin/dig +edns=0 . SOA'
    """
    def __init__(self, context):
        super(DigEdns, self).__init__(context, '/usr/bin/dig +edns=0 . SOA')


@parser(Specs.dig_noedns)
class DigNoedns(Dig):
    """
    Class for parsing ``/usr/bin/dig +noedns . SOA`` command.

    Sample output of this command is::

        ; <<>> DiG 9.11.1-P3-RedHat-9.11.1-3.P3.fc26 <<>> +noedns . SOA
        ;; global options: +cmd
        ;; Got answer:
        ;; ->>HEADER<<- opcode: QUERY, status: NOERROR, id: 47135
        ;; flags: qr rd ra ad; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 0

        ;; QUESTION SECTION:
        ;.				IN	SOA

        ;; ANSWER SECTION:
        .			20195	IN	SOA	a.root-servers.net. nstld.verisign-grs.com. 2017120600 1800 900 604800 86400

        ;; Query time: 22 msec
        ;; SERVER: 10.38.5.26#53(10.38.5.26)
        ;; WHEN: Thu Dec 07 09:31:24 CET 2017
        ;; MSG SIZE  rcvd: 92

    Examples:
        >>> dig_noedns.status
        'NOERROR'
        >>> dig_noedns.has_signature
        False
        >>> dig_noedns.command
        '/usr/bin/dig +noedns . SOA'
    """
    def __init__(self, context):
        super(DigNoedns, self).__init__(context, '/usr/bin/dig +noedns . SOA')



from __future__ import print_function
import six
import sys

from datetime import datetime
from insights import dr, rule
from insights.util import utc


RENDERERS = {}
_FORMATTERS = {}


def get_formatter(name):
    """
    Looks up a formatter class given a prefix to it.
    The names are sorted, and the first matching class is returned.
    """
    for k in sorted(_FORMATTERS):
        if k.startswith(name):
            return _FORMATTERS[k]


class FormatterAdapterMeta(type):
    """ Automatically registers subclasses for later lookup. """

    def __init__(cls, name, bases, dct):
        if name not in ("FormatterAdapter", "EvaluatorFormatterAdapter"):
            _FORMATTERS[dr.get_name(cls)] = cls
        super(FormatterAdapterMeta, cls).__init__(name, bases, dct)


class FormatterAdapter(six.with_metaclass(FormatterAdapterMeta)):

    @staticmethod
    def configure(p):
        """ Override to add arguments to the ArgumentParser. """
        pass

    def __init__(self, args=None):
        """ Subclasses get passed the parsed args. """
        pass

    def preprocess(self, broker):
        """
        Called before any components have been run. Useful for registering
        observers.
        """
        pass

    def postprocess(self, broker):
        """
        Called after all components have been run. Useful for interrogating
        the broker for final state.
        """


class Formatter(object):
    def __init__(self, broker, stream=sys.stdout):
        self.broker = broker
        self.stream = stream
        self.start_time = datetime.now(utc)

    def __enter__(self):
        self.preprocess()
        return self

    def __exit__(self, _type, value, tb):
        self.postprocess()

    def preprocess(self):
        pass

    def postprocess(self):
        pass


class EvaluatorFormatterAdapter(FormatterAdapter):
    """
    Base class for formatters that want to serialize a SingleEvaluator after
    execution.
    """
    Impl = None

    @staticmethod
    def configure(p):
        """ Override to add arguments to the ArgumentParser. """
        p.add_argument("-F", "--fail-only", help="Show FAIL results only. Conflict with '-m' or '-f', will be dropped when using them together", action="store_true")

    def __init__(self, args=None):
        if args:
            hn = "insights.combiners.hostname, insights.parsers.branch_info"
            args.plugins = ",".join([args.plugins, hn]) if args.plugins else hn
            if args.fail_only:
                print('Options conflict: -f and -F, drops -F', file=sys.stderr)
                args.fail_only = False

    def preprocess(self, broker):
        self.formatter = self.Impl(broker)
        self.formatter.preprocess()

    def postprocess(self, broker):
        self.formatter.postprocess()


def get_content(obj, val):
    """
    Attempts to determine a jinja2 content template for a rule's response.
    """
    # does the rule define a content= kwarg?
    c = dr.get_delegate(obj).content

    # otherwise, does the rule module have a CONTENT attribute?
    if c is None:
        mod = sys.modules[obj.__module__]
        c = getattr(mod, "CONTENT", None)

    if c:
        # is the content a dictionary?
        if isinstance(c, dict):

            # does it contain a make_* class as a key?
            v = c.get(val.__class__)
            if v is not None:
                return v

            # does it contain an error key?
            key = val.get_key()
            if key:
                v = c.get(key)

                # is the value a dict that contains make_* classes?
                if isinstance(v, dict):
                    return v.get(val.__class__)
                return v
        else:
            return c


try:
    from jinja2 import Template

    def format_rule(comp, val):
        content = get_content(comp, val)
        if content and val.get("type") != "skip":
            return Template(content).render(val)
        return str(val)

    RENDERERS[rule] = format_rule
except:
    pass


def render(comp, val):
    _type = dr.get_component_type(comp)
    func = RENDERERS.get(_type)
    return func(comp, val) if func else str(val)



"""
CorosyncCmapctl - Command ``corosync-cmapctl [params]``
=======================================================

This module parses the output of the ``corosync-cmapctl [params]`` command.
"""

from insights import parser, CommandParser
from insights.parsers import SkipException, ParseException
from insights.specs import Specs


@parser(Specs.corosync_cmapctl)
class CorosyncCmapctl(CommandParser, dict):
    """
    Class for parsing the `/usr/sbin/corosync-cmapctl [params]` command.
    All lines are stored in the dictionary with the left part of the equal
    sign witout parenthese info as the key and the right part of equal sign
    as the value.

    Typical output of the command is::

        config.totemconfig_reload_in_progress (u8) = 0
        internal_configuration.service.0.name (str) = corosync_cmap
        internal_configuration.service.0.ver (u32) = 0
        internal_configuration.service.1.name (str) = corosync_cfg
        internal_configuration.service.1.ver (u32) = 0
        internal_configuration.service.2.name (str) = corosync_cpg
        internal_configuration.service.2.ver (u32) = 0

    Examples:
        >>> type(corosync)
        <class 'insights.parsers.corosync_cmapctl.CorosyncCmapctl'>
        >>> 'internal_configuration.service.0.name' in corosync
        True
        >>> corosync['internal_configuration.service.0.name']
        'corosync_cmap'

    Raises:
        SkipException: When there is no content
        ParseException: When there is no "=" in the content
    """

    def __init__(self, context):
        super(CorosyncCmapctl, self).__init__(context, extra_bad_lines=['corosync-cmapctl: invalid option'])

    def parse_content(self, content):
        if not content:
            raise SkipException
        for line in content:
            if '=' not in line:
                raise ParseException("Can not parse line %s" % line)
            key, value = [item.strip() for item in line.split('=')]
            key_without_parenthese = key.split()[0]
            self[key_without_parenthese] = value



from insights.tests import context_wrap
from insights.parsers.sysconfig import PrelinkSysconfig

PRELINK_SYSCONFIG = """
# Set this to no to disable prelinking altogether
# (if you change this from yes to no prelink -ua
# will be run next night to undo prelinking)
PRELINKING=no

# Options to pass to prelink
# -m    Try to conserve virtual memory by allowing overlapping
#       assigned virtual memory slots for libraries which
#       never appear together in one binary
# -R    Randomize virtual memory slot assignments for libraries.
#       This makes it slightly harder for various buffer overflow
#       attacks, since library addresses will be different on each
#       host using -R.
PRELINK_OPTS=-mR

# How often should full prelink be run (in days)
# Normally, prelink will be run in quick mode, every
# $PRELINK_FULL_TIME_INTERVAL days it will be run
# in normal mode.  Comment it out if it should be run
# in normal mode always.
PRELINK_FULL_TIME_INTERVAL=14

# How often should prelink run (in days) even if
# no packages have been upgraded via rpm.
# If $PRELINK_FULL_TIME_INTERVAL days have not elapsed
# yet since last normal mode prelinking, last
# quick mode prelinking happened less than
# $PRELINK_NONRPM_CHECK_INTERVAL days ago
# and no packages have been upgraded by rpm
# since last quick mode prelinking, prelink
# will not do anything.
# Change to
# PRELINK_NONRPM_CHECK_INTERVAL=0
# if you want to disable the rpm database timestamp
# check (especially if you don't use rpm/up2date/yum/apt-rpm
# exclusively to upgrade system libraries and/or binaries).
PRELINK_NONRPM_CHECK_INTERVAL=7
""".strip()


def test_sysconfig_prelink():
    result = PrelinkSysconfig(context_wrap(PRELINK_SYSCONFIG))
    assert result["PRELINKING"] == 'no'
    assert result.get("PRELINK_OPTS") == '-mR'
    assert result.get("OPTIONS1") is None
    assert result['PRELINK_NONRPM_CHECK_INTERVAL'] == "7"



from insights.parsers.ls_var_log import LsVarLog
from insights.tests import context_wrap
from insights.util.file_permissions import FilePermissions

# from RHEL 7.2
LS_1 = """
/var/log:
total 1016
drwxr-xr-x.  7 root root   4096 Oct 19 15:38 .
drwxr-xr-x. 19 root root   4096 Oct 19 15:38 ..
drwxr-xr-x.  2 root root   4096 Jun 28 10:26 anaconda
drwxr-x---.  2 root root     22 Jun 28 10:26 audit
-rw-r--r--.  1 root root   8433 Oct 19 15:38 boot.log
-rw-------.  1 root utmp      0 Oct 19 05:39 btmp
-rw-------.  1 root root  11370 Oct 19 22:01 cron
-rw-r--r--.  1 root root  32226 Oct 19 15:38 dmesg
-rw-r--r--.  1 root root  34423 Oct 19 15:27 dmesg.old
-rw-------.  1 root root   5466 Oct 19 05:40 grubby
-rw-r--r--.  1 root root 292292 Oct 19 22:42 lastlog
-rw-------.  1 root root   1386 Oct 19 15:39 maillog
-rw-------.  1 root root 609617 Oct 19 22:42 messages
drwx------.  2 root root      6 Jan 26  2014 ppp
drwxr-xr-x.  2 root root     41 Jun 28 10:28 rhsm
-rw-------.  1 root root  18185 Oct 19 22:42 secure
-rw-------.  1 root root      0 Jun 28 10:20 spooler
-rw-------.  1 root root      0 Jun 28 10:19 tallylog
drwxr-xr-x.  2 root root     22 Sep  1 07:32 tuned
-rw-r--r--.  1 root root   1854 Oct 19 22:38 up2date
-rw-r--r--.  1 root root 211784 Jun 28 10:41 vboxadd-install.log
-rw-r--r--.  1 root root     73 Oct 19 05:17 vboxadd-install-x11.log
-rw-r--r--.  1 root root      1 Jun 28 10:40 VBoxGuestAdditions.log
-rw-r--r--.  1 root root    280 Oct 19 15:38 wpa_supplicant.log
-rw-rw-r--.  1 root utmp  21504 Oct 19 22:42 wtmp
-rw-------.  1 root root   8423 Oct 19 15:29 yum.log

/var/log/anaconda:
total 1048
drwxr-xr-x. 2 root root   4096 Jun 28 10:26 .
drwxr-xr-x. 7 root root   4096 Oct 19 15:38 ..
-rw-------. 1 root root  17862 Jun 28 10:26 anaconda.log
-rw-------. 1 root root   1726 Jun 28 10:26 ifcfg.log
-rw-------. 1 root root 680224 Jun 28 10:26 journal.log
-rw-------. 1 root root      0 Jun 28 10:26 ks-script-2aaku7.log
-rw-------. 1 root root 114262 Jun 28 10:26 packaging.log
-rw-------. 1 root root  30137 Jun 28 10:26 program.log
-rw-------. 1 root root  88243 Jun 28 10:26 storage.log
-rw-------. 1 root root  63190 Jun 28 10:26 syslog
-rw-------. 1 root root  52756 Jun 28 10:26 X.log

/var/log/audit:
total 1288
drwxr-x---. 2 root root      22 Jun 28 10:26 .
drwxr-xr-x. 7 root root    4096 Oct 19 15:38 ..
-rw-------. 1 root root 1259137 Oct 19 22:42 audit.log

/var/log/ppp:
total 4
drwx------. 2 root root    6 Jan 26  2014 .
drwxr-xr-x. 7 root root 4096 Oct 19 15:38 ..

/var/log/rhsm:
total 32
drwxr-xr-x. 2 root root    41 Jun 28 10:28 .
drwxr-xr-x. 7 root root  4096 Oct 19 15:38 ..
-rw-r--r--. 1 root root  4279 Oct 19 19:39 rhsmcertd.log
-rw-r--r--. 1 root root 18331 Oct 19 19:39 rhsm.log

/var/log/tuned:
total 16
drwxr-xr-x. 2 root root   22 Sep  1 07:32 .
drwxr-xr-x. 7 root root 4096 Oct 19 15:38 ..
-rw-r--r--. 1 root root 8834 Oct 19 15:39 tuned.log

""".strip()


"""
These tests are mainly derived from the old implementation of LsVarLog,
which did its own parsing and used FilePermissions objects throughout.
The new implementation used the more completely tested and more robust
FileListing parser.  So some tests are a bit ... odd.
"""


def test_smoketest():
    context = context_wrap(LS_1)
    result = LsVarLog(context)

    assert "/var/log" in result
    assert "/var/log/audit" in result
    assert "/var/log/ppp" in result
    assert "/var/log/rhsm" in result
    assert "/var/log/tuned" in result
    assert "audit.log" in result.listings["/var/log/audit"]['entries']
    assert "audit.log" == result.get_filepermissions("/var/log/audit", "audit.log").path


def test_dir_parsed():
    context = context_wrap(LS_1)
    result = LsVarLog(context)
    ls = {}
    current_dir = ""
    test_lines = LS_1.splitlines()
    for line in test_lines:
        # wonky parsing from memory to test the actual implementation; doesn't expect evil input!
        if line.endswith(":"):
            current_dir = line.split(":")[0]
            ls[current_dir] = []
        elif line.startswith("total"):
            pass
        elif line:
            fileperm = FilePermissions(line)
            ls[current_dir].append(fileperm.path)
    for dir in ls:
        assert dir in result
        dir_from_parser = result.listings[dir]['entries']
        # Two way cross-check:
        # Were all files in our simple parse found in the parser?
        for fil in ls[dir]:
            assert fil in dir_from_parser
        # Were all files in the parser found in our simple parse?

        for fil in dir_from_parser:
            assert fil in ls[dir]


def test_get_filepermissions():
    context = context_wrap(LS_1)
    result = LsVarLog(context)
    ls = {}
    current_dir = ""
    test_lines = LS_1.splitlines()
    for line in test_lines:
        # wonky parsing from memory to test the actual implementation; doesn't expect evil input!
        if line.endswith(":"):
            current_dir = line.split(":")[0]
            ls[current_dir] = []
        elif line.startswith("total"):
            pass
        elif line:
            fileperm = FilePermissions(line)
            ls[current_dir].append(fileperm)
    for dir in ls:
        assert dir in result
        for fil in ls[dir]:
            found = result.get_filepermissions(dir, fil.path)
            assert found is not None
            assert fil.line == found.line
            not_found = result.get_filepermissions(dir, "nonexisting" + fil.path)
            assert not_found is None



from insights.parsers import docker_host_machine_id
from insights.tests import context_wrap

DOCKER_HOST_MACHINE_ID = """
e6637bbb-ae92-46f8-a249-92d184c5fc24
"""


def test_docker_host_machine_id():
    machine_id = docker_host_machine_id.docker_host_machineid_parser(context_wrap(DOCKER_HOST_MACHINE_ID))
    assert machine_id.get("host_system_id") == "e6637bbb-ae92-46f8-a249-92d184c5fc24"



import doctest

from insights.parsers import ls_var_tmp
from insights.parsers.ls_var_tmp import LsVarTmp
from insights.tests import context_wrap

LS_VAR_TMP = """
/var/tmp:
total 20
drwxr-xr-x.  2 0 0 4096 Mar 26 02:25 a1
drwxr-xr-x.  2 0 0 4096 Mar 26 02:25 a2
drwxr-xr-x.  3 0 0 4096 Apr  3 02:50 foreman-ssh-cmd-fc3f65c9-2b35-480d-87e3-1d971433d6ad
"""


def test_ls_var_tmp():
    ls_var_tmp = LsVarTmp(context_wrap(LS_VAR_TMP))
    assert ls_var_tmp.dirs_of('/var/tmp') == ['a1', 'a2', 'foreman-ssh-cmd-fc3f65c9-2b35-480d-87e3-1d971433d6ad']
    foreman = ls_var_tmp.dir_entry('/var/tmp', 'foreman-ssh-cmd-fc3f65c9-2b35-480d-87e3-1d971433d6ad')
    assert foreman is not None
    assert foreman == {
        'group': '0',
        'name': 'foreman-ssh-cmd-fc3f65c9-2b35-480d-87e3-1d971433d6ad',
        'links': 3,
        'perms': 'rwxr-xr-x.',
        'raw_entry': 'drwxr-xr-x.  3 0 0 4096 Apr  3 02:50 foreman-ssh-cmd-fc3f65c9-2b35-480d-87e3-1d971433d6ad',
        'owner': '0',
        'date': 'Apr  3 02:50',
        'type': 'd',
        'size': 4096,
        'dir': '/var/tmp'}


def test_ls_var_tmp_doc_examples():
    env = {
        'LsVarTmp': LsVarTmp,
        'ls_var_tmp': LsVarTmp(context_wrap(LS_VAR_TMP)),
    }
    failed, total = doctest.testmod(ls_var_tmp, globs=env)
    assert failed == 0



from insights.parsers.current_clocksource import CurrentClockSource
from insights.tests import context_wrap

CLKSRC = """
tsc
"""


def test_get_current_clksr():
    clksrc = CurrentClockSource(context_wrap(CLKSRC))
    assert clksrc.data == "tsc"
    assert clksrc.is_kvm is False
    assert clksrc.is_vmi_timer != clksrc.is_tsc



"""
SoftNetStats - file ``/proc/net/softnet_stat``
==============================================

This parser parses the stats from network devices. These stats includes events
per cpu(in row), number of packets processed i.e packet_process (first column),
number of packet drops packet_drops (second column), time squeeze i.e net_rx_action
performed time_squeeze(third column), cpu collision i.e collision occur while obtaining
device lock while transmitting cpu_collision packets (eighth column), received_rps
number of times cpu woken up received_rps (ninth column), number of times reached flow
limit count flow_limit_count (tenth column).

The typical contents of ``cat /proc/net/softnet_stat`` file is as below::

    00008e78 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
    000040ee 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
    0001608c 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
    0000372f 00000001 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000

    Column-01: packet_process: Packet processed by each CPU.
    Column-02: packet_drop: Packets dropped.
    Column-03: time_squeeze: net_rx_action.
    Column-08: cpu_collision: collision occur while obtaining device lock while transmitting.
    Column-09: received_rps: number of times cpu woken up received_rps.
    Column-10: flow_limit_count: number of times reached flow limit count.

.. Note::
            There is minimal documentation about these fields in the file, columns are not labeled and
            could change between the kernel releases. This format is compatible for RHEL-6
            and RHEL-7 releases. Also it is unlikely that the positions of those values will
            change in short term.

Examples:
    >>> SOFTNET_STATS = '''
    ... 00008e78 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
    ... 000040ee 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
    ... 0001608c 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
    ... 0000372f 00000001 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000
    ...'''.strip()
    >>> softnet_obj = SoftNetStats(context_wrap(SOFTNET_STATS))
    >>> softnet_obj.is_packet_drops()
    True
    >>> softnet_obj.cpu_instances
    4
    >>> softnet_obj.per_cpu_nstat('packet_drops')
    [0, 0, 0, 1]
"""

from .. import Parser, parser
from insights.specs import Specs


@parser(Specs.softnet_stat)
class SoftNetStats(Parser):
    """Parses ``/proc/net/softnet_stat`` file contains"""

    def __init__(self, context):

        """ Total number of CPUs listed in the stats records"""
        self.cpu_instances = 0

        """ List of network stats per cpu instace """
        self.cpu_nstats = []
        super(SoftNetStats, self).__init__(context)

    def parse_content(self, contents):
        for line in contents:
            stats = [int(st, 16) for st in line.split(None)]
            self.cpu_nstats.append({'packet_process': stats[0],
                            'packet_drops': stats[1],
                            'time_squeeze': stats[2],
                            'cpu_collision': stats[7],
                            'received_rps': stats[8],
                            'flow_limit_count': stats[9]})
        self.cpu_instances = len(self.cpu_nstats)

    def per_cpu_nstat(self, key):
        """
        Get network stats per column for all cpu.

        Arguments:
            (str): Column name for eg. packet_drops or received_rps.

        Returns:
            (list): Column states per cpu.
        """
        if not self.cpu_nstats:
            return []

        if key not in self.cpu_nstats[0]:
            return []

        return [cpu[key] for cpu in self.cpu_nstats]

    @property
    def is_packet_drops(self):
        """
        It will check for if there is packet drop occurred on any cpu.

        Arguments:
            None: No input argument for the function

        Returns:
            (bool): It will return True if observed packet drops.
        """
        return any(cpu["packet_drops"] > 0 for cpu in self.cpu_nstats)



"""
AwxManage - commands ``awx-manage``
===================================

Parsers contains in this module are:

AnsibleTowerLicenseType - command ``awx-manage check_license``
--------------------------------------------------------------
"""

from insights import JSONParser, parser, CommandParser
from insights.parsers import SkipException, ParseException
from insights.specs import Specs


@parser(Specs.awx_manage_check_license)
class AnsibleTowerLicenseType(CommandParser, JSONParser):
    """
    Parses the output of command  ``awx-manage check_license``

    Sample output of the command::

        enterprise

    Attributes:
        type (str): The license type, e.g. "enterprise"

    Examples:
    >>> type(awx_license)
    <class 'insights.parsers.awx_manage.AnsibleTowerLicenseType'>
    >>> awx_license.type == "enterprise"
    True
    """
    def parse_content(self, content):
        if not content:
            raise SkipException
        if len(content) != 1:
            raise ParseException("Invalid output: {0}".format(content))
        self.type = content[0].strip()



"""
Simultaneous Multithreading (SMT) combiner
==========================================

Combiner for Simultaneous Multithreading (SMT). It uses the results of the following parsers:
:class:`insights.parsers.smt.CpuCoreOnline`,
:class:`insights.parsers.smt.CpuSiblings`.
"""

from insights.core.plugins import combiner
from insights.parsers.smt import CpuCoreOnline, CpuSiblings


@combiner(CpuCoreOnline, CpuSiblings)
class CpuTopology(object):
    """
    Class for collecting the online/siblings status for all CPU cores.

    Sample output of the ``CpuCoreOnline`` parser is::

        [[Core 0: Online], [Core 1: Online], [Core 2: Online], [Core 3: Online]]

    Sample output of the ``CpuSiblings`` parser is::

        [[Core 0 Siblings: [0, 2]], [Core 1 Siblings: [1, 3]], [Core 2 Siblings: [0, 2]], [Core 3 Siblings: [1, 3]]]

    Attributes:
        cores (list of dictionaries): List of all cores.
        all_solitary (bool): True, if hyperthreading is not used.

    Examples:
        >>> type(cpu_topology)
        <class 'insights.combiners.smt.CpuTopology'>
        >>> cpu_topology.cores == [{'online': True, 'siblings': [0, 2]}, {'online': True, 'siblings': [1, 3]}, {'online': True, 'siblings': [0, 2]}, {'online': True, 'siblings': [1, 3]}]
        True
        >>> cpu_topology.all_solitary
        False
    """

    def __init__(self, cpu_online, cpu_siblings):
        self.cores = []

        max_cpu_core_id = max([core.core_id for core in cpu_online])
        for n in range(max_cpu_core_id + 1):
            online = [core for core in cpu_online if core.core_id == n]
            online = online[0].on
            siblings = [sibling for sibling in cpu_siblings if sibling.core_id == n]
            if len(siblings) != 0:
                siblings = siblings[0].siblings

            one_core = {"online": online, "siblings": siblings}
            self.cores.append(one_core)

        self.all_solitary = all([len(core["siblings"]) <= 1 for core in self.cores])

    def online(self, core_id):
        """
        Returns bool value obtained from "online" file for given core_id.
        """
        if core_id >= len(self.cores) or core_id < 0:
            return None
        return self.cores[core_id]["online"]

    def siblings(self, core_id):
        """
        Returns list of siblings for given core_id.
        """
        if core_id >= len(self.cores) or core_id < 0:
            return None
        return self.cores[core_id]["siblings"]



"""
YumConf - file ``/etc/yum.conf``
================================

This module provides parsing for the ``/etc/yum.conf`` file.
The ``YumConf`` class parses the information in the file
``/etc/yum.conf``. See the ``IniConfigFile`` class for more
information on attributes and methods.

Sample input data looks like::

    [main]

    cachedir=/var/cache/yum/$basearch/$releasever
    keepcache=0
    debuglevel=2
    logfile=/var/log/yum.log
    exactarch=1
    obsoletes=1
    gpgcheck=1
    plugins=1
    installonly_limit=3

    [rhel-7-server-rpms]

    metadata_expire = 86400
    baseurl = https://cdn.redhat.com/content/rhel/server/7/$basearch
    name = Red Hat Enterprise Linux 7 Server (RPMs)
    gpgkey = file:///etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release
    enabled = 1
    gpgcheck = 1

Examples:

    >>> yconf = shared[YumConf]
    >>> yconf.defaults()
    {'admin_token': 'ADMIN', 'compute_port': '8774'}
    >>> 'main' in yconf
    True
    >>> 'rhel-7-server-rpms' in yconf
    True
    >>> yconf.has_option('main', 'gpgcheck')
    True
    >>> yconf.has_option('main', 'foo')
    False
    >>> yconf.get('rhel-7-server-rpms', 'enabled')
    '1'
    >>> yconf.items('main')
    {'plugins': '1',
     'keepcache': '0',
     'cachedir': '/var/cache/yum/$basearch/$releasever',
     'exactarch': '1',
     'obsoletes': '1',
     'installonly_limit': '3',
     'debuglevel': '2',
     'gpgcheck': '1',
     'logfile': '/var/log/yum.log'}
"""

import six
from insights.contrib.ConfigParser import NoOptionError
from .. import parser, IniConfigFile
from insights.specs import Specs


@parser(Specs.yum_conf)
class YumConf(IniConfigFile):
    """Parse contents of file ``/etc/yum.conf``."""
    def parse_content(self, content):
        super(YumConf, self).parse_content(content)
        # File /etc/yum.conf may contain repos definitions.
        # Keywords 'gpgkey' and 'baseurl' might contain multiple
        # values separated by comma. Convert those values into a list.
        for section in self.sections():
            for key in ('gpgkey', 'baseurl'):
                try:
                    value = self.get(section, key)
                    if value and isinstance(value, six.string_types):
                        self.data.set(section, key, value.split(','))
                except NoOptionError:
                    pass



#!/usr/bin/env python
"""
Given a list of plugins to load, print a report of any containing rules that
might produce the same result keys.
"""
from __future__ import print_function
import argparse
import inspect
from collections import defaultdict

from insights import dr, parse_plugins, rule


def parse_args():
    p = argparse.ArgumentParser(description="Find rule modules that could produce the same response keys.")
    p.add_argument("-p",
                   "--plugins",
                   default="",
                   help="Comma separated list of package(s) or module(s) containing plugins.")
    return p.parse_args()


def main():
    args = parse_args()

    plugins = parse_plugins(args.plugins)
    for p in plugins:
        dr.load_components(p, continue_on_error=False)

    results = defaultdict(list)
    for comp, delegate in dr.DELEGATES.items():
        if isinstance(delegate, rule):
            results[dr.get_base_module_name(comp)].append(comp)

    results = dict((key, comps) for key, comps in results.items() if len(comps) > 1)

    if results:
        print("Potential key conflicts:")
        print()
        for key in sorted(results):
            print("{key}:".format(key=key))
            for comp in sorted(results[key], key=dr.get_name):
                name = comp.__name__
                path = inspect.getabsfile(comp)
                print("    {name} in {path}".format(name=name, path=path))
            print()


if __name__ == "__main__":
    main()



import doctest

from insights.combiners import package_provides_httpd
from insights.parsers.package_provides_httpd import PackageProvidesHttpd
from insights.combiners.package_provides_httpd import PackageProvidesHttpdAll
from insights.tests import context_wrap


PACKAGE_COMMAND_MATCH_1 = """
/opt/rh/httpd24/root/usr/sbin/httpd httpd24-httpd-2.4.34-7.el7.x86_64
"""

PACKAGE_COMMAND_MATCH_2 = """
/usr/sbin/httpd httpd-2.4.6-88.el7.x86_64
"""

PACKAGE_COMMAND_MATCH_3 = """
/opt/rh/jbcs-httpd24/root/usr/sbin/httpd jbcs-httpd24-httpd-2.4.34-7.el7.x86_64
"""


def test_packages_provide_httpd():
    pack1 = PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_MATCH_1))
    pack2 = PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_MATCH_2))
    pack3 = PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_MATCH_3))
    result = PackageProvidesHttpdAll([pack1, pack2, pack3])
    assert sorted(result.running_httpds) == sorted(
        ['/opt/rh/httpd24/root/usr/sbin/httpd',
         '/usr/sbin/httpd', '/opt/rh/jbcs-httpd24/root/usr/sbin/httpd'])
    assert result["/usr/sbin/httpd"] == "httpd-2.4.6-88.el7.x86_64"
    assert result.get_package("/opt/rh/httpd24/root/usr/sbin/httpd") == "httpd24-httpd-2.4.34-7.el7.x86_64"
    assert result.get("/opt/rh/httpd24/root/usr/sbin/httpd") == "httpd24-httpd-2.4.34-7.el7.x86_64"
    assert result.get_package("/usr/lib/httpd") is None
    assert result.get("/usr/lib/httpd") is None


def test_doc_examples():
    pack1 = PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_MATCH_1))
    pack2 = PackageProvidesHttpd(context_wrap(PACKAGE_COMMAND_MATCH_2))
    env = {
        'packages': package_provides_httpd.PackageProvidesHttpdAll([pack1, pack2]),
    }
    failed, _ = doctest.testmod(package_provides_httpd, globs=env)
    assert failed == 0



from insights import combiner, dr
from insights.core.context import HostContext
from insights.parsers.uname import Uname


def teardown_function(*args):
    for k in dr.ENABLED:
        dr.ENABLED[k] = True


@combiner()
def one():
    return 1


def test_enabled_string():
    assert dr.is_enabled("insights.core.context.HostContext")


def test_enabled_object():
    assert dr.is_enabled(Uname)


def test_set_enabled_object():
    dr.set_enabled(HostContext)
    assert dr.is_enabled(HostContext)


def test_disabled_string():
    dr.set_enabled("insights.core.context.HostContext", False)
    assert not dr.is_enabled(HostContext)


def test_disabled_object():
    dr.set_enabled(Uname, False)
    assert not dr.is_enabled(Uname)


def test_disabled_run():
    assert dr.ENABLED[one]
    broker = dr.run(dr.COMPONENTS[dr.GROUPS.single])
    assert one in broker

    dr.set_enabled(one, False)
    assert not dr.ENABLED[one]
    broker = dr.run(dr.COMPONENTS[dr.GROUPS.single])
    assert one not in broker



#!/usr/bin/env python

from __future__ import print_function

from insights import rule, make_metadata, run
from insights.specs import Specs
from insights.core import Parser
from insights.core.plugins import parser
from insights.core.dr import set_enabled
import uuid


def valid_uuid_or_None(s):
    """
    Returns  if `s` is a valid UUID string.
    """
    if not s:
        return None
    try:
        return str(uuid.UUID(s))
    except Exception:
        return None


@parser(Specs.ip_addresses)
class IPs(Parser):
    """
    Reads the output of hostname -I and constructs a list of all assigned IP
    addresses.

    Example output::
       192.168.1.71 10.88.0.1 172.17.0.1 172.18.0.1 10.10.121.131 2600:1700:720:7e30:e4ef:e9d0:7ea1:c8a7

    Resultant data structure::
        [
            "192.168.1.71",
            "10.88.0.1",
            "172.17.0.1",
            "172.18.0.1",
            "10.10.121.131",
            "2600:1700:720:7e30:e4ef:e9d0:7ea1:c8a7"
        ]
    """

    def parse_content(self, content):
        self.data = content[0].rstrip().split()


@parser(Specs.subscription_manager_id)
class SubscriptionManagerID(Parser):
    """
    Reads the output of subscription-manager identity and retrieves the UUID

    Example output::
        system identity: 6655c27c-f561-4c99-a23f-f53e5a1ef311
        name: rhel7.localdomain
        org name: 1234567
        org ID: 1234567

    Resultant data::

        6655c27c-f561-4c99-a23f-f53e5a1ef311
    """

    def parse_content(self, content):
        self.data = content[0].split(":")[-1].strip()


def _safe_parse(ds):
    try:
        return ds.content[0]

    except Exception:
        return None


def _filter_falsy(dict_):
    return dict((k, v) for k, v in dict_.items() if v)


@rule(
    optional=[
        Specs.machine_id,
        Specs.etc_machine_id,
        Specs.bios_uuid,
        SubscriptionManagerID,
        IPs,
        Specs.hostname,
        Specs.mac_addresses,
    ]
)
def canonical_facts(
    insights_id, machine_id, bios_uuid, submanid, ips, fqdn, mac_addresses
):

    facts = dict(
        insights_id=valid_uuid_or_None(_safe_parse(insights_id)),
        machine_id=valid_uuid_or_None(_safe_parse(machine_id)),
        bios_uuid=valid_uuid_or_None(_safe_parse(bios_uuid)),
        subscription_manager_id=valid_uuid_or_None(submanid.data if submanid else None),
        ip_addresses=ips.data if ips else [],
        mac_addresses=list(
            filter(None, (_safe_parse(c) for c in mac_addresses))
        ) if mac_addresses else [],
        fqdn=_safe_parse(fqdn),
    )

    return make_metadata(**_filter_falsy(facts))


def get_canonical_facts(path=None):
    set_enabled(canonical_facts, True)
    set_enabled(SubscriptionManagerID, True)
    set_enabled(IPs, True)
    br = run(canonical_facts, root=path)
    d = br[canonical_facts]
    del d["type"]
    return d


if __name__ == "__main__":
    import json
    from insights import dr
    dr.load_components("insights.specs.default", "insights.specs.insights_archive")

    print(json.dumps(get_canonical_facts()))



import doctest
from insights.parsers import version_info
from insights.tests import context_wrap


VER_INFO_1 = """
{"core_version": "3.0.8-dev", "client_version": "3.1.1"}
""".strip()

VER_INFO_2 = """
{"core_version": "3.0.203-1", "client_version": "3.1.1"}
""".strip()


def test_version_info():
    ret = version_info.VersionInfo(context_wrap(VER_INFO_1))
    assert ret.core_version == '3.0.8-dev'
    assert ret.client_version == '3.1.1'

    ret = version_info.VersionInfo(context_wrap(VER_INFO_2))
    assert ret.core_version == '3.0.203-1'
    assert ret.client_version == '3.1.1'


def test_doc_examples():
    env = {
        'ver': version_info.VersionInfo(context_wrap(VER_INFO_2)),
    }
    failed, total = doctest.testmod(version_info, globs=env)
    assert failed == 0



"""
LsDev - Command ``ls -lanR /dev``
=================================

The ``ls -lanR /dev`` command provides information for the listing of the
``/dev`` directory.

Sample input is shown in the Examples. See ``FileListing`` class for
additional information.

Examples:
    >>> LS_DEV = '''
    ... /dev:
    ... total 3
    ... brw-rw----.  1 0  6 253,   0 Aug  4 16:56 dm-0
    ... brw-rw----.  1 0  6 253,   1 Aug  4 16:56 dm-1
    ... brw-rw----.  1 0  6 253,  10 Aug  4 16:56 dm-10
    ... crw-rw-rw-.  1 0  5   5,   2 Aug  5  2016 ptmx
    ... drwxr-xr-x.  2 0  0        0 Aug  4 16:56 pts
    ... lrwxrwxrwx.  1 0  0       25 Oct 25 14:48 initctl -> /run/systemd/initctl/fifo
    ...
    ... /dev/rhel:
    ... total 0
    ... drwxr-xr-x.  2 0 0  100 Jul 25 10:00 .
    ... drwxr-xr-x. 23 0 0 3720 Jul 25 12:43 ..
    ... lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 home -> ../dm-2
    ... lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 root -> ../dm-0
    ... lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 swap -> ../dm-1
    ...'''
    >>> ls_dev = LsDev(context_wrap(LS_DEV))
    >>> ls_dev
    <insights.parsers.ls_dev.LsDev object at 0x7f287406a1d0>
    >>> "/dev/rhel" in ls_dev
    True
    >>> ls_dev.files_of("/dev/rhel")
    ['home', 'root', 'swap']
    >>> ls_dev.dirs_of("/dev/rhel")
    ['.', '..']
    >>> ls_dev.specials_of("/dev/rhel")
    []
    >>> ls_dev.listing_of("/dev/rhel").keys()
    ['home', 'root', 'swap', '..', '.']
    >>> ls_dev.dir_entry("/dev/rhel", "home")
    {'group': '0', 'name': 'home', 'links': 1, 'perms': 'rwxrwxrwx.',
    'raw_entry': 'lrwxrwxrwx.  1 0 0    7 Jul 25 10:00 home -> ../dm-2', 'owner': '0',
    'link': '../dm-2', 'date': 'Jul 25 10:00', 'type': 'l', 'size': 7}
    >>> ls_dev.listing_of('/dev/rhel')['.']['type'] == 'd'
    True
    >>> ls_dev.listing_of('/dev/rhel')['home']['link']
    '../dm-2'
"""
from .. import parser, FileListing, CommandParser
from insights.specs import Specs


@parser(Specs.ls_dev)
class LsDev(CommandParser, FileListing):
    """Parses output of ``ls -lanR /dev`` command."""
    pass



"""
PackageProvidesHttpd - command ``/bin/echo {httpd_command_package}``
====================================================================

.. warning::

    This module is deprecated, please use
    :mod:`insights.parsers.package_provides` instead.

This module parses the content that contains running instances of 'httpd' and
its corresponding RPM package which provide them. The running command and its
package name are stored as properties ``command`` and ``package`` of the object.

The reason why using above datasource is that we need to record the links
between running_httpd_command and package which provides the httpd command. In
``ps aux`` output, we can only get what httpd command starts a httpd
application, instead of httpd package. Through this way, when there is httpd
bug, we can detect whether a running httpd application will be affected.

Examples:
    >>> package.command
    '/opt/rh/httpd24/root/usr/sbin/httpd'
    >>> package.package
    'httpd24-httpd-2.4.34-7.el7.x86_64'
"""

from insights import parser, CommandParser
from insights.specs import Specs
from ..parsers import SkipException
from insights.util import deprecated


@parser(Specs.package_provides_httpd)
class PackageProvidesHttpd(CommandParser):
    """
    .. warning::

        This Combiner is deprecated, please use
        :class:`insights.parsers.package_provides.PackageProvidesCommand` instead.

    Parse the content like '/opt/rh/httpd24/root/usr/sbin/httpd /usr/sbin/httpd'

    Attributes:
        command (str): The httpd command that starts application.
        package (str): httpd package that provides above httpd command.
    """

    def parse_content(self, content):
        deprecated(
            PackageProvidesHttpd,
            'Please use the :class:`insights.parsers.package_provides.PackageProvidesCommand` instead.'
        )
        if len(content) == 0:
            raise SkipException("Error: ", 'there is not httpd application running')
        l = content[0].split()
        if len(l) != 2:
            raise SkipException("Error: ",
                                'current running httpd command is not provided by package installed through yum or rpm')
        self.command = l[0]
        self.package = l[1]



import datetime
from insights.parsers.uptime import Uptime
from insights.parsers.facter import Facter
from insights.combiners.uptime import uptime
from insights.tests import context_wrap

UPTIME1 = " 14:28:24 up  5:55,  4 users,  load average: 0.04, 0.03, 0.05"
UPTIME2 = " 10:55:22 up 40 days, 3 min,  1 user,  load average: 0.49, 0.12, 0.04"
UPTIME3 = """
COMMAND> facts
uptime => 21 days
uptime_days => 21
uptime_hours => 525
uptime_seconds => 1893598
""".strip()


def total_seconds(time_delta):
    return (time_delta.days * 24 * 60 * 60) + time_delta.seconds


def test_get_uptime_uptime1():
    ut = Uptime(context_wrap(UPTIME1))
    upt = uptime(ut, None)
    assert upt.currtime == '14:28:24'
    assert upt.updays == ""
    assert upt.uphhmm == '5:55'
    assert upt.users == '4'
    assert upt.loadavg == ['0.04', '0.03', '0.05']
    c = datetime.timedelta(days=0, hours=5, minutes=55)
    assert total_seconds(upt.uptime) == total_seconds(c)


def test_get_uptime_uptime2():
    ut = Uptime(context_wrap(UPTIME2))
    upt = uptime(ut, None)
    assert upt.currtime == '10:55:22'
    assert upt.updays == '40'
    assert upt.uphhmm == '00:03'
    assert upt.users == '1'
    assert upt.loadavg == ['0.49', '0.12', '0.04']
    c = datetime.timedelta(days=40, hours=0, minutes=3)
    assert total_seconds(upt.uptime) == total_seconds(c)


def test_get_facter_uptime():
    ft = Facter(context_wrap(UPTIME3))
    upt = uptime(None, ft)
    assert upt.updays == "21"
    assert upt.uphhmm == '21:59'
    assert upt.loadavg is None
    c = datetime.timedelta(days=0, hours=0, minutes=0, seconds=1893598)
    assert total_seconds(upt.uptime) == total_seconds(c)


def test_get_both_uptime():
    ut = Uptime(context_wrap(UPTIME2))
    ft = Facter(context_wrap(UPTIME3))
    upt = uptime(ut, ft)
    assert upt.currtime == '10:55:22'
    assert upt.updays == '40'
    assert upt.uphhmm == '00:03'
    assert upt.users == '1'
    assert upt.loadavg == ['0.49', '0.12', '0.04']
    c = datetime.timedelta(days=40, hours=0, minutes=3)
    assert total_seconds(upt.uptime) == total_seconds(c)



from insights.tests import context_wrap
from insights.parsers.grub_conf import Grub1Config, Grub2Config

# RHEL7
GRUB2_CFG_1 = """
## BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server, with Linux 3.10.0-123.9.3.el7.x86_64' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.9.3.el7.x86_64-advanced-5a1c841f-5cfe-4d59-b3a0-4b788369d6cb' {
    load_video
    set gfxpayload=keep
    insmod gzio
    insmod part_msdos
    insmod ext2
    set root='hd0,msdos1'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  93f2c0dc-6201-4d26-a610-4b3998983ea2
    else
      search --no-floppy --fs-uuid --set=root 93f2c0dc-6201-4d26-a610-4b3998983ea2
    fi
    linux16 /vmlinuz-3.10.0-123.9.3.el7.x86_64 root=UUID=5a1c841f-5cfe-4d59-b3a0-4b788369d6cb ro crashkernel=auto  vconsole.font=latarcyrheb-sun16 console=ttyS0,38400 rd.lvm.lv=VG_RACS-CCP/lv-rootfs vconsole.keymap=us LANG=en_US.UTF-8
    initrd16 /initramfs-3.10.0-123.9.3.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server, with Linux 0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3-advanced-5a1c841f-5cfe-4d59-b3a0-4b788369d6cb' {
    load_video
    insmod gzio
    insmod part_msdos
    insmod ext2
    set root='hd0,msdos1'
    if [ x$feature_platform_search_hint = xy ]; then
      search --no-floppy --fs-uuid --set=root --hint-bios=hd0,msdos1 --hint-efi=hd0,msdos1 --hint-baremetal=ahci0,msdos1 --hint='hd0,msdos1'  93f2c0dc-6201-4d26-a610-4b3998983ea2
    else
      search --no-floppy --fs-uuid --set=root 93f2c0dc-6201-4d26-a610-4b3998983ea2
    fi
    linux16 /vmlinuz-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3 root=UUID=5a1c841f-5cfe-4d59-b3a0-4b788369d6cb ro crashkernel=auto  vconsole.font=latarcyrheb-sun16 console=ttyS0,38400 rd.lvm.lv=VG_RACS-CCP/lv-rootfs vconsole.keymap=us
    initrd16 /initramfs-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3.img
}
""".strip()

LS_BOOT_1 = """
/boot:
total 68392
dr-xr-xr-x.  4 0 0     4096 Jan 12 17:45 .
drwxr-xr-x. 19 0 0     4096 Jan 13 19:49 ..
-rw-r--r--.  1 0 0      170 Oct 30 09:50 .vmlinuz-3.10.0-123.9.3.el7.x86_64.hmac
-rw-------.  1 0 0  2841475 Oct 30 09:50 System.map-3.10.0-123.9.3.el7.x86_64
-rw-r--r--.  1 0 0   122093 Oct 30 09:50 config-3.10.0-123.9.3.el7.x86_64
drwxr-xr-x.  6 0 0     4096 Jan 12 17:45 grub2
-rw-r--r--.  1 0 0 38922434 Jan 12 17:45 initramfs-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3.img
-rw-r--r--.  1 0 0 17364252 Jan 12 17:45 initramfs-3.10.0-123.9.3.el7.x86_64.img
-rw-r--r--.  1 0 0   589451 Jan 12 17:45 initrd-plymouth.img
drwx------.  2 0 0    16384 Jan 12 17:38 lost+found
-rw-r--r--.  1 0 0   228621 Oct 30 09:52 symvers-3.10.0-123.9.3.el7.x86_64.gz
-rwxr-xr-x.  1 0 0  4904112 Jan 12 17:45 vmlinuz-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3
-rwxr-xr-x.  1 0 0  4904112 Oct 30 09:50 vmlinuz-3.10.0-123.9.3.el7.x86_64
""".strip()

# RHEL7
GRUB2_CFG_2 = """
### BEGIN /etc/grub.d/10_linux ###
menuentry 'Red Hat Enterprise Linux Server (3.10.0-229.el7.x86_64) 7.0 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.el7.x86_64-advanced-7f18fec3-a016-42ab-9bb9-6e7f6a5985ca' {
    linux16 /vmlinuz-3.10.0-229.el7.x86_64 root=/dev/mapper/rhel-root ro rd.lvm.lv=rhel/root crashkernel=auto  rd.lvm.lv=rhel/swap vconsole.font=latarcyrheb-sun16 vconsole.keymap=us rhgb quiet LANG=en_AU.UTF-8
    initrd16 /initramfs-3.10.0-229.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server (3.10.0-123.13.2.el7.x86_64) 7.0 (Maipo)' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.el7.x86_64-advanced-7f18fec3-a016-42ab-9bb9-6e7f6a5985ca' {
    linux16 /vmlinuz-3.10.0-123.13.2.el7.x86_64 root=/dev/mapper/rhel-root ro rd.lvm.lv=rhel/root crashkernel=auto  rd.lvm.lv=rhel/swap vconsole.font=latarcyrheb-sun16 vconsole.keymap=us rhgb quiet LANG=en_AU.UTF-8
    initrd16 /initramfs-3.10.0-123.13.2.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server, with Linux 3.10.0-123.el7.x86_64' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-3.10.0-123.el7.x86_64-advanced-7f18fec3-a016-42ab-9bb9-6e7f6a5985ca' {
    linux16 /vmlinuz-3.10.0-123.el7.x86_64 root=UUID=7f18fec3-a016-42ab-9bb9-6e7f6a5985ca ro rd.lvm.lv=rhel/root crashkernel=auto  rd.lvm.lv=rhel/swap vconsole.font=latarcyrheb-sun16 vconsole.keymap=us rhgb quiet LANG=en_AU.UTF-8
    initrd16 /initramfs-3.10.0-123.el7.x86_64.img
}
menuentry 'Red Hat Enterprise Linux Server, with Linux 0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad' --class red --class gnu-linux --class gnu --class os --unrestricted $menuentry_id_option 'gnulinux-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad-advanced-7f18fec3-a016-42ab-9bb9-6e7f6a5985ca' {
    linux16 /vmlinuz-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad root=UUID=7f18fec3-a016-42ab-9bb9-6e7f6a5985ca ro rd.lvm.lv=rhel/root crashkernel=auto  rd.lvm.lv=rhel/swap vconsole.font=latarcyrheb-sun16 vconsole.keymap=us rhgb quiet
    initrd16 /initramfs-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad.img
}
""".strip()

LS_BOOT_2 = """
/boot:
total 173260
dr-xr-xr-x.  3 0 0     4096 Mar 26 12:23 .
drwxr-xr-x. 17 0 0     4096 Mar 26 11:12 ..
-rw-r--r--.  1 0 0      171 Dec 13 10:54 .vmlinuz-3.10.0-123.13.2.el7.x86_64.hmac
-rw-r--r--.  1 0 0      166 May  6  2014 .vmlinuz-3.10.0-123.el7.x86_64.hmac
-rw-r--r--.  1 0 0      166 Jan 30 09:40 .vmlinuz-3.10.0-229.el7.x86_64.hmac
-rw-------.  1 0 0  2841409 Dec 13 10:54 System.map-3.10.0-123.13.2.el7.x86_64
-rw-------.  1 0 0  2840084 May  6  2014 System.map-3.10.0-123.el7.x86_64
-rw-------.  1 0 0  2881257 Jan 30 09:40 System.map-3.10.0-229.el7.x86_64
-rw-r--r--.  1 0 0   122094 Dec 13 10:54 config-3.10.0-123.13.2.el7.x86_64
-rw-r--r--.  1 0 0   122059 May  6  2014 config-3.10.0-123.el7.x86_64
-rw-r--r--.  1 0 0   123838 Jan 30 09:40 config-3.10.0-229.el7.x86_64
drwxr-xr-x.  6 0 0      104 Mar 26 12:05 grub2
-rw-r--r--.  1 0 0 40654410 Dec 22 14:48 initramfs-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad.img
-rw-------.  1 0 0 16742742 Dec 22 15:36 initramfs-3.10.0-123.13.2.el7.x86_64.img
-rw-------.  1 0 0 18795253 Mar 26 12:02 initramfs-3.10.0-123.13.2.el7.x86_64kdump.img
-rw-------.  1 0 0 17246244 Dec 22 14:52 initramfs-3.10.0-123.el7.x86_64.img
-rw-------.  1 0 0 17111790 Dec 22 15:34 initramfs-3.10.0-123.el7.x86_64kdump.img
-rw-------.  1 0 0 18180132 Mar 26 12:05 initramfs-3.10.0-229.el7.x86_64.img
-rw-------.  1 0 0 18400548 Mar 26 12:23 initramfs-3.10.0-229.el7.x86_64kdump.img
-rw-r--r--.  1 0 0   867063 Mar 26 12:05 initrd-plymouth.img
-rw-r--r--.  1 0 0   228603 Dec 13 10:55 symvers-3.10.0-123.13.2.el7.x86_64.gz
-rw-r--r--.  1 0 0   228562 May  6  2014 symvers-3.10.0-123.el7.x86_64.gz
-rw-r--r--.  1 0 0   240039 Jan 30 09:44 symvers-3.10.0-229.el7.x86_64.gz
-rwxr-xr-x.  1 0 0  4902000 Dec 22 14:48 vmlinuz-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad
-rwxr-xr-x.  1 0 0  4904688 Dec 13 10:54 vmlinuz-3.10.0-123.13.2.el7.x86_64
-rwxr-xr-x.  1 0 0  4902000 May  6  2014 vmlinuz-3.10.0-123.el7.x86_64
-rwxr-xr-x.  1 0 0  5026624 Jan 30 09:40 vmlinuz-3.10.0-229.el7.x86_64
""".strip()

# RHEL5/6
GRUB_CONF_3 = """
# grub.conf generated by anaconda
#
# Note that you do not have to rerun grub after making changes to this file
# NOTICE:  You have a /boot partition.  This means that
#          all kernel and initrd paths are relative to /boot/, eg.
#          root (hd0,0)
#          kernel /vmlinuz-version ro root=/dev/cciss/c0d0p3
#          initrd /initrd-version.img
#boot=/dev/cciss/c0d0
default=1
# fallback=0 # commented out by Proliant HBA install script
timeout=5
splashimage=(hd0,0)/grub/splash.xpm.gz
hiddenmenu
title Red Hat Enterprise Linux Server (2.6.18-194.8.1.el5)
    root (hd0,0)
    kernel /vmlinuz-2.6.18-194.8.1.el5 ro root=LABEL=/1 crashkernel=128M@16M
    initrd /initrd-2.6.18-194.8.1.el5.img

title Red Hat Enterprise Linux Server (2.6.18-194.17.1.el5)
    root (hd0,0)
    kernel /vmlinuz-2.6.18-194.17.1.el5 ro root=LABEL=/1 crashkernel=128M@16M
    initrd /initrd-2.6.18-194.17.1.el5.img
""".strip()

# Kernel and initrd referenced in grub.conf are actually softlinks to other files
GRUB_CONF_LINKS = """
title Linux (Logical Disk 0)
        root (hd0,0)
        kernel /boot/vmlinuz ro root=LABEL=/root LANG=en_US.UTF-8 SYSFONT=latarcyrheb-sun16 KEYBOARDTYPE=pc KEYTABLE=us crashkernel=768M  rd_NO_PLYMOUTH nodmraid
        initrd /boot/initrd
""".strip()
# Xen configs look a little different in grub.conf
GRUB_CONF_XEN = """
title Red Hat Enterprise Linux Server (2.6.18-398.el5xen)
    root (hd0,0)
    kernel /xen.gz-2.6.18-398.el5
    module /vmlinuz-2.6.18-398.el5xen ro root=/dev/sda2
    module /initrd-2.6.18-398.el5xen.img
""".strip()
# Some machines PXE boot so don't check them because we can't know if they will work or not
GRUB_CONF_IPXE = """
title iPXE
  root (hd0,0)
  kernel (hd0,0)/boot/ipxe.lkrn
""".strip()


def test_get_grub_kernel_initrd_1():
    expected_result = {'grub_kernels': ["vmlinuz-2.6.18-194.8.1.el5", "vmlinuz-2.6.18-194.17.1.el5"],
                       'grub_initrds': ["initrd-2.6.18-194.8.1.el5.img", "initrd-2.6.18-194.17.1.el5.img"]}
    assert expected_result == Grub1Config(context_wrap(GRUB_CONF_3)).kernel_initrds

    # Process grub.conf where kernel and initrd are softlinks
    expected_result = {'grub_kernels': ['vmlinuz'], 'grub_initrds': ['initrd']}
    assert expected_result == Grub1Config(context_wrap(GRUB_CONF_LINKS)).kernel_initrds

    # Process grub.conf from a Xen hypervisor
    expected_result = {'grub_kernels': ['vmlinuz-2.6.18-398.el5xen'], 'grub_initrds': ['initrd-2.6.18-398.el5xen.img']}
    assert expected_result == Grub1Config(context_wrap(GRUB_CONF_XEN)).kernel_initrds

    # Simulate grub.conf not containing any kernel or initrd entries
    expected_result = {'grub_kernels': [], 'grub_initrds': []}
    assert expected_result == Grub1Config(context_wrap(LS_BOOT_1)).kernel_initrds
    assert expected_result, Grub1Config(context_wrap(LS_BOOT_2)).kernel_initrds

    # Ignore grub.conf from machines that have PXE boot entries because we can't know if they are ok or not
    assert Grub1Config(context_wrap(GRUB_CONF_IPXE)).kernel_initrds == {}


def test_get_grub_kernel_initrd_2():
    expected_result = {'grub_kernels': ["vmlinuz-3.10.0-123.9.3.el7.x86_64", "vmlinuz-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3"],
                       'grub_initrds': ["initramfs-3.10.0-123.9.3.el7.x86_64.img", "initramfs-0-rescue-00c2fbfaa85544e48d6ca1d919fa2dd3.img"]}
    assert expected_result == Grub2Config((context_wrap(GRUB2_CFG_1))).kernel_initrds == expected_result
    expected_result = {'grub_kernels': ["vmlinuz-3.10.0-229.el7.x86_64", "vmlinuz-3.10.0-123.13.2.el7.x86_64",
                                        "vmlinuz-3.10.0-123.el7.x86_64", "vmlinuz-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad"],
                       'grub_initrds': ["initramfs-3.10.0-229.el7.x86_64.img", "initramfs-3.10.0-123.13.2.el7.x86_64.img",
                                        "initramfs-3.10.0-123.el7.x86_64.img", "initramfs-0-rescue-13798ffcbc1ed4374f3f2e0fa6c923ad.img"]}
    assert expected_result == Grub2Config(context_wrap(GRUB2_CFG_2)).kernel_initrds

    # Simulate grub.conf not containing any kernel or initrd entries
    expected_result = {'grub_kernels': [], 'grub_initrds': []}
    assert expected_result == Grub2Config(context_wrap(LS_BOOT_1)).kernel_initrds
    assert expected_result, Grub2Config(context_wrap(LS_BOOT_2)).kernel_initrds



"""
CephConf - file ``/etc/ceph/ceph.conf``
=======================================

The CephConf class parses the file ``/etc/ceph/ceph.conf``. The
ceph.conf is in the standard 'ini' format and is read by the base
parser class `IniConfigFile`.

Sample ``/etc/ceph/ceph.conf`` file::

    [global]
    osd_pool_default_pgp_num = 128
    auth_service_required = cephx
    mon_initial_members = controller1-az1,controller1-az2,controller1-az3
    fsid = a4c3-11e8-99c5-5254003f2830-ea8796d6
    cluster_network = 10.xx.xx.xx/23
    auth_supported = cephx
    auth_cluster_required = cephx
    mon_host = 10.xx.xx.xx,xx.xx.xx.xx,10.xx.xx.xx
    auth_client_required = cephx
    osd_pool_default_size = 3
    osd_pool_default_pg_num = 128
    ms_bind_ipv6 = false
    public_network = 10.xx.xx.xx/23

    [osd]
    osd_journal_size = 81920

    [mon.controller1-az2]
    public_addr = 10.xx.xx.xx

    [client.radosgw.gateway]
    user = apache
    rgw_frontends = civetweb port=10.xx.xx.xx:8080
    log_file = /var/log/ceph/radosgw.log
    host = controller1-az2
    keyring = /etc/ceph/ceph.client.radosgw.gateway.keyring
    rgw_keystone_implicit_tenants = true
    rgw_keystone_token_cache_size = 500
    rgw_keystone_url = http://10.xx.xx.xx:35357
    rgw_s3_auth_use_keystone = true
    rgw_keystone_admin_467fE = Xqzta6dYhPHGHGEFaGnctoken
    rgw_keystone_accepted_roles = admin,_member_,Member
    rgw_swift_account_in_url = true


Examples:

    >>> list(conf.sections()) == ['global', 'osd', 'mon.controller1-az2', 'client.radosgw.gateway']
    True
    >>> conf.has_option('osd', 'osd_journal_size')
    True
    >>> conf.getboolean('client.radosgw.gateway', 'rgw_swift_account_in_url')
    True
    >>> conf.get('client.radosgw.gateway', 'rgw_swift_account_in_url') == 'true'
    True
    >>> conf.get('client.radosgw.gateway', 'user') == 'apache'
    True
    >>> conf.get('client.radosgw.gateway', 'log_file') == '/var/log/ceph/radosgw.log'
    True
"""
from .. import IniConfigFile, parser, add_filter
from insights.specs import Specs

add_filter(Specs.ceph_conf, ["["])


@parser(Specs.ceph_conf)
class CephConf(IniConfigFile):
    """Class for ceph.conf file content."""
    pass
